from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} & {{last_name}}",
        "{{company_prefix}} {{last_name}}",
        "{{large_company}}",
    )

    company_prefixes = (
        "Stichting",
        "Koninklijke",
        "Royal",
    )

    company_suffixes = (
        "BV",
        "NV",
        "Groep",
    )

    # Source: https://www.mt.nl/management/reputatie/mt-500-2018-de-lijst/559930
    large_companies = (
        "Shell",
        "Coolblue",
        "ASML",
        "Ahold",
        "Tata Steel",
        "KLM",
        "Bol.com",
        "BP Nederland",
        "De Efteling",
        "Eneco",
        "De Persgroep",
        "ING",
        "Royal HaskoningDHV",
        "Randstad",
        "Google",
        "Ikea",
        "Rockwool",
        "BAM",
        "Achmea",
        "Damen Shipyard",
        "ABN Amro",
        "Remeha Group",
        "TenneT",
        "Coca-Cola",
        "Van Leeuwen Buizen",
        "Wavin",
        "Rabobank",
        "AkzoNobel",
        "Arcadis",
        "AFAS",
        "Cisco",
        "DAF Trucks",
        "DHL",
        "Hanos",
        "Boon Edam",
        "BMW Nederland",
        "The Greenery",
        "Dutch Flower Group",
        "Koninklijke Mosa",
        "Yacht",
        "Rituals",
        "Microsoft",
        "Esso",
        "3W Vastgoed",
        "Deloitte",
        "Corio",
        "Voortman Steel Group",
        "Agrifirm",
        "Makro Nederland",
        "Nederlandse Publieke Omroep",
        "De Alliantie",
        "Heijmans",
        "McDonalds",
        "ANWB",
        "Mediamarkt",
        "Kruidvat" "Van Merksteijn Steel",
        "Dura Vermeer",
        "Alliander",
        "Unilever",
        "Enexis",
        "Berenschot",
        "Jumbo",
        "Technische Unie",
        "Havenbedrijf Rotterdam",
        "Ballast Nedam",
        "RTL Nederland",
        "Talpa Media",
        "Blauwhoed Vastgoed",
        "DSM",
        "Ymere",
        "Witteveen+Bos",
        "NS",
        "Action",
        "FloraHolland",
        "Heineken",
        "Nuon",
        "EY",
        "Dow Benelux",
        "Bavaria",
        "Schiphol",
        "Holland Casino",
        "Binck bank",
        "BDO",
        "HEMA",
        "Alphabet Nederland",
        "Croon Elektrotechniek",
        "ASR Vastgoed ontwikkeling",
        "PwC",
        "Mammoet",
        "KEMA",
        "IBM",
        "A.S. Watson",
        "KPMG",
        "VodafoneZiggo",
        "YoungCapital",
        "Triodos Bank",
        "Aviko",
        "AgruniekRijnvallei",
        "Heerema",
        "Accenture",
        "Aegon",
        "NXP",
        "Breman Installatiegroep",
        "Movares Groep",
        "Q-Park",
        "FleuraMetz",
        "Sanoma",
        "Bakker Logistiek",
        "VDL Group",
        "Bayer",
        "Boskalis",
        "Nutreco",
        "Dell",
        "Brunel",
        "Exact",
        "Manpower",
        "Essent",
        "Canon",
        "ONVZ Zorgverzekeraar",
        "Telegraaf Media Group",
        "Nationale Nederlanden",
        "Andus Group",
        "Den Braven Group",
        "ADP",
        "ASR",
        "ArboNed",
        "Plieger",
        "De Heus Diervoeders",
        "USG People",
        "Bidvest Deli XL",
        "Apollo Vredestein",
        "Tempo-Team",
        "Trespa",
        "Janssen Biologics",
        "Starbucks",
        "PostNL",
        "Vanderlande",
        "FrieslandCampina",
        "Constellium",
        "Huisman",
        "Abbott",
        "Koninklijke Boom Uitgevers",
        "Bosch Rexroth",
        "BASF",
        "Audax",
        "VolkerWessels",
        "Hunkemöller",
        "Athlon Car Lease",
        "DSW Zorgverzekeraar",
        "Mars",
        "De Brauw Blackstone Westbroek",
        "NDC Mediagroep",
        "Bluewater",
        "Stedin",
        "Feenstra",
        "Wuppermann Staal Nederland",
        "Kramp",
        "SABIC",
        "Iv-Groep",
        "Bejo Zaden",
        "Wolters Kluwer",
        "Nyrstar holding",
        "Adecco",
        "Tauw",
        "Robeco",
        "Eriks",
        "Allianz Nederland Groep",
        "Driessen",
        "Burger King",
        "Lekkerland",
        "Van Lanschot",
        "Brocacef",
        "Bureau Veritas",
        "Relx",
        "Pathé Bioscopen",
        "Bosal",
        "Ardagh Group",
        "Maandag",
        "Inalfa",
        "Atradius",
        "Capgemini",
        "Greenchoice",
        "Q8 (Kuwait Petroleum Europe)",
        "ASM International",
        "Van der Valk",
        "Delta Lloyd",
        "GlaxoSmithKline",
        "ABB",
        "Fabory, a Grainger company",
        "Veen Bosch & Keuning Uitgeversgroep",
        "CZ",
        "Plus",
        "RET Rotterdam",
        "Loyens & Loeff",
        "Holland Trading",
        "Archer Daniels Midland Nederland",
        "Ten Brinke",
        "NAM",
        "DAS",
        "Samsung Electronics Benelux",
        "Koopman International",
        "TUI",
        "Lannoo Meulenhoff",
        "AC Restaurants",
        "Stage Entertainment",
        "Acer",
        "HDI Global SE",
        "Detailresult",
        "Nestle",
        "GVB Amsterdam",
        "Dekamarkt",
        "Dirk",
        "MSD",
        "Arriva",
        "Baker Tilly Berk",
        "SBM Offshore",
        "TomTom",
        "Fujifilm",
        "B&S",
        "BCC",
        "Gasunie",
        "Oracle Nederland",
        "Astellas Pharma",
        "SKF",
        "Woningstichting Eigen Haard",
        "Rijk Zwaan",
        "Chubb",
        "Fugro",
        "Total",
        "Rochdale",
        "ASVB",
        "Atos",
        "Acomo",
        "KPN",
        "Van Drie Group",
        "Olympia uitzendbureau",
        "Bacardi Nederland",
        "JMW Horeca Uitzendbureau",
        "Warner Bros/Eyeworks",
        "Aalberts Industries",
        "SNS Bank",
        "Amtrada Holding",
        "VGZ",
        "Grolsch",
        "Office Depot",
        "De Rijke Group",
        "Bovemij Verzekeringsgroep",
        "Coop Nederland",
        "Eaton Industries",
        "ASN",
        "Yara Sluiskil",
        "HSF Logistics",
        "Fokker",
        "Deutsche Bank",
        "Sweco",
        "Univé Groep",
        "Koninklijke Wagenborg",
        "Strukton",
        "Conclusion",
        "Philips",
        "In Person",
        "Fluor",
        "Vroegop-Windig",
        "ArboUnie",
        "Centraal Boekhuis",
        "Siemens",
        "Connexxion",
        "Fujitsu",
        "Consolid",
        "AVR Afvalverwerking",
        "Brabant Alucast",
        "Centric",
        "Havensteder",
        "Novartis",
        "Booking.com",
        "Menzis",
        "Frankort & Koning Groep",
        "Jan de Rijk",
        "Brand Loyalty Group",
        "Ohra Verzekeringen",
        "Terberg Group",
        "Cloetta",
        "Holland & Barrett",
        "Enza Zaden",
        "VION",
        "Woonzorg Nederland",
        "T-Mobile",
        "Crucell",
        "NautaDutilh",
        "BNP Paribas",
        "NIBC Bank",
        "VastNed",
        "CCV Holland",
        "IHC Merwede",
        "Neways",
        "NSI N.V.",
        "Deen",
        "Accor",
        "HTM",
        "ITM Group",
        "Ordina",
        "Dümmen Orange",
        "Optiver",
        "Zara",
        "L'Oreal Nederland B.V.",
        "Vinci Energies",
        "Suit Supply Topco",
        "Sita",
        "Vos Logistics",
        "Altran",
        "St. Clair",
        "BESI",
        "Fiat Chrysler Automobiles",
        "UPS",
        "Jacobs",
        "Emté",
        "TBI",
        "De Bijenkorf",
        "Aldi Nederland",
        "Van Wijnen",
        "Vitens",
        "De Goudse Verzekeringen",
        "SBS Broadcasting",
        "Sandd",
        "Omron",
        "Sogeti",
        "Alfa Accountants & Adviseurs",
        "Harvey Nash",
        "Stork",
        "Glencore Grain",
        "Meijburg & Co",
        "Honeywell",
        "Meyn",
        "Ericsson Telecommunicatie",
        "Hurks",
        "Mitsubishi",
        "GGN",
        "CGI Nederland",
        "Staples Nederland",
        "Denkavit International",
        "Ecorys",
        "Rexel Nederland",
        "A. Hakpark",
        "DuPont Nederland",
        "CBRE Group",
        "Bolsius",
        "Marel",
        "Metro",
        "Flynth Adviseurs en Accountants",
        "Kropman Installatietechniek",
        "Kuijpers",
        "Medtronic",
        "Cefetra",
        "Simon Loos",
        "Citadel Enterprises",
        "Intergamma",
        "Ceva Logistics",
        "Beter Bed",
        "Subway",
        "Gamma",
        "Karwei" "Varo Energy",
        "APM Terminals",
        "Center Parcs",
        "Brenntag Nederland",
        "NFI",
        "Hoogvliet",
        "Van Gansewinkel",
        "Nedap",
        "Blokker",
        "Perfetti Van Melle",
        "Vestia",
        "Kuehne + Nagel Logistics",
        "Rensa Group",
        "NTS Group",
        "Joh. Mourik & Co. Holding",
        "Mercedes-Benz",
        "DIT Personeel",
        "Verkade",
        "Hametha",
        "Vopak",
        "IFF",
        "Pearle",
        "Mainfreight",
        "De Jong & Laan",
        "DSV",
        "P4People",
        "Mazars",
        "Cargill",
        "Ten Brinke Groep",
        "Alewijnse",
        "Agio Cigars",
        "Peter Appel Transport",
        "Syngenta",
        "Avery Dennison",
        "Accon AVM",
        "Vitol",
        "Vermaat Groep",
        "BMC",
        "Alcatel-Lucent",
        "Maxeda DIY",
        "Equens",
        "Van Gelder Groep",
        "Emerson Electric Nederland",
        "Bakkersland",
        "Specsavers",
        "E.On",
        "Landal Greenparks",
        "IMC Trading",
        "Barentz Group",
        "Epson",
        "Raet",
        "Van Oord",
        "Thomas Cook Nederland",
        "SDU uitgevers",
        "Nedschroef",
        "Linde Gas",
        "Ewals Cargo Care",
        "Theodoor Gilissen",
        "TMF Group",
        "Cornelis Vrolijk",
        "Jan Linders Supermarkten",
        "SIF group",
        "BT Nederland",
        "Kinepolis",
        "Pink Elephant",
        "General Motors Nederland",
        "Carlson Wagonlit",
        "Bruna",
        "Docdata",
        "Schenk Tanktransport",
        "WPG",
        "Peak-IT",
        "Martinair",
        "Reesink",
        "Elopak Nederland",
        "Fagron N.V.",
        "OVG Groep",
        "Ford Nederland",
        "Multi Corporation",
        "Simac",
        "Primark",
        "Tech Data Nederland",
        "Vleesgroothandel Zandbergen",
        "Raben Group",
        "Farm Frites",
        "Libéma",
        "Caldic",
        "Portaal",
        "Syntus",
        "Jacobs DE",
        "Stena Line",
        "The Phone House",
        "Interfood Group",
        "Thales",
        "Teva Pharmaceuticals",
        "RFS Holland",
        "Aebi Schmidt Nederland",
        "Rockwell Automation Nederland",
        "Engie Services",
        "Hendrix Genetics",
        "Qbuzz",
        "Unica",
        "2SistersFoodGroup",
        "Ziut",
        "Munckhof Groep",
        "Spar Holding",
        "Samskip",
        "Continental Bakeries",
        "Sligro",
        "Merck",
        "Foot Locker Europe",
        "Unit4",
        "PepsiCo",
        "Sulzer",
        "Tebodin",
        "Value8",
        "Boels",
        "DKG Groep",
        "Bruynzeel Keukens",
        "Janssen de Jong Groep",
        "ProRail",
        "Solid Professionals",
        "Hermes Partners",
    )

    def large_company(self) -> str:
        """
        :example: 'Bol.com'
        """
        return self.random_element(self.large_companies)

    def company_prefix(self) -> str:
        """
        :example: 'Stichting'
        """
        return self.random_element(self.company_prefixes)



from .. import Provider as AddressProvider


class Provider(AddressProvider):
    address_formats = ["{{street_address}}, {{city_name}}, {{postcode}}"]
    building_number_formats = ["#", "##", "###"]
    city_formats = ["{{city_prefix}} {{city_name}}"]
    street_address_formats = (
        "{{street_prefix}} {{street_name}}, буд. {{building_number}}",
        "{{street_prefix}} {{street_name}}, буд. {{building_number}} кв. {{building_number}}",
    )

    street_name_formats = "{{street_prefix}} {{street_title}}"

    city_prefixes = ["місто", "село", "селище", "хутір"]

    city_names = (
        "Авдіївка",
        "Алмазна",
        "Алупка",
        "Алушта",
        "Алчевськ",
        "Амвросіївка",
        "Ананьїв",
        "Андрушівка",
        "Антрацит ",
        "Апостолове",
        "Армянськ",
        "Арциз",
        "Балаклія",
        "Балта",
        "Бар",
        "Баранівка",
        "Барвінкове",
        "Батурин",
        "Бахмач",
        "Бахмут",
        "Бахчисарай",
        "Баштанка",
        "Белз",
        "Бердичів",
        "Бердянськ",
        "Берегове",
        "Бережани",
        "Березань ",
        "Березівка",
        "Березне",
        "Берестечко",
        "Берислав",
        "Бершадь",
        "Бібрка",
        "Біла Церква",
        "Білгород-Дністровський",
        "Білицьке",
        "Білогірськ",
        "Білозерське",
        "Білопілля",
        "Біляївка",
        "Благовіщенське",
        "Бобринець",
        "Бобровиця",
        "Богодухів",
        "Богуслав",
        "Боково-Хрустальне",
        "Болград",
        "Болехів",
        "Борзна",
        "Борислав",
        "Бориспіль",
        "Борщів",
        "Боярка",
        "Бровари",
        "Броди",
        "Брянка",
        "Бунге ",
        "Буринь",
        "Бурштин ",
        "Буськ",
        "Буча",
        "Бучач",
        "Валки ",
        "Вараш",
        "Василівка ",
        "Васильків",
        "Ватутіне",
        "Вашківці",
        "Великі Мости",
        "Верхівцеве",
        "Верхньодніпровськ",
        "Вижниця",
        "Вилкове",
        "Винники",
        "Виноградів",
        "Вишгород",
        "Вишневе ",
        "Вільногірськ",
        "Вільнянськ",
        "Вінниця",
        "Вовчанськ",
        "Вознесенівка",
        "Вознесенськ",
        "Волноваха",
        "Володимир-Волинський",
        "Волочиськ",
        "Ворожба",
        "Вуглегірськ",
        "Вугледар",
        "Гадяч",
        "Гайворон",
        "Гайсин",
        "Галич",
        "Генічеськ",
        "Географія Вільнянська",
        "Герца",
        "Гірник ",
        "Гірське",
        "Глиняни",
        "Глобине",
        "Глухів",
        "Гнівань",
        "Гола Пристань",
        "Голубівка",
        "Горішні Плавні",
        "Горлівка",
        "Городенка",
        "Городище ",
        "Городня",
        "Городок",
        "Горохів",
        "Гребінка",
        "Гуляйполе",
        "Дебальцеве",
        "Деражня",
        "Дергачі",
        "Джанкой",
        "Дніпро ",
        "Дніпрорудне",
        "Добромиль",
        "Добропілля",
        "Довжанськ",
        "Докучаєвськ",
        "Долина ",
        "Долинська",
        "Донецьк",
        "Дрогобич",
        "Дружба ",
        "Дружківка",
        "Дубляни",
        "Дубно",
        "Дубровиця",
        "Дунаївці",
        "Енергодар",
        "Євпаторія",
        "Єнакієве",
        "Жашків",
        "Жданівка",
        "Жидачів",
        "Житомир",
        "Жмеринка",
        "Жовква",
        "Жовті Води",
        "Заводське",
        "Залізне",
        "Заліщики",
        "Запоріжжя",
        "Заставна",
        "Збараж",
        "Зборів",
        "Звенигородка",
        "Здолбунів",
        "Зеленодольськ",
        "Зимогір'я",
        "Зіньків",
        "Зміїв",
        "Знам'янка",
        "Золоте",
        "Золотоноша",
        "Золочів",
        "Зоринськ",
        "Зугрес",
        "Івано-Франківськ",
        "Ізмаїл",
        "Ізюм",
        "Ізяслав",
        "Іллінці",
        "Іловайськ",
        "Інкерман",
        "Ірміно",
        "Ірпінь",
        "Іршава",
        "Ічня",
        "Кагарлик",
        "Кадіївка",
        "Калинівка ",
        "Калуш",
        "Кальміуське",
        "Кам'янець-Подільський",
        "Кам'янка (Черкаська область)",
        "Кам'янка-Бузька",
        "Кам'янка-Дніпровська",
        "Кам'янське",
        "Камінь-Каширський",
        "Канів",
        "Карлівка",
        "Каховка",
        "Керч",
        "Київ",
        "Кипуче",
        "Ківерці",
        "Кілія",
        "Кіцмань",
        "Кобеляки",
        "Ковель",
        "Кодима",
        "Козятин",
        "Коломия",
        "Комарно",
        "Конотоп",
        "Копичинці",
        "Корець",
        "Коростень",
        "Коростишів",
        "Корсунь-Шевченківський",
        "Корюківка",
        "Косів",
        "Костопіль",
        "Костянтинівка",
        "Краматорськ",
        "Красилів",
        "Красногорівка",
        "Красноград",
        "Красноперекопськ",
        "Кременець",
        "Кременчук",
        "Кремінна",
        "Кривий Ріг",
        "Кролевець",
        "Кропивницький",
        "Куп'янськ",
        "Курахове",
        "Ладижин",
        "Ланівці",
        "Лебедин",
        "Лиман ",
        "Липовець",
        "Лисичанськ",
        "Лозова ",
        "Лохвиця",
        "Лубни",
        "Луганськ",
        "Лутугине",
        "Луцьк",
        "Львів",
        "Любомль",
        "Люботин",
        "Макіївка",
        "Мала Виска",
        "Малин",
        "Мар'їнка",
        "Марганець ",
        "Маріуполь",
        "Мелітополь",
        "Мена ",
        "Мерефа",
        "Миколаїв",
        "Миколаївка ",
        "Миргород",
        "Мирноград",
        "Миронівка",
        "Міусинськ",
        "Могилів-Подільський",
        "Молодогвардійськ",
        "Молочанськ",
        "Монастириська",
        "Монастирище",
        "Моршин",
        "Моспине",
        "Мостиська",
        "Мукачево",
        "Надвірна",
        "Немирів",
        "Нетішин",
        "Ніжин",
        "Нікополь",
        "Нова Каховка",
        "Нова Одеса",
        "Новгород-Сіверський",
        "Нове Давидково",
        "Новий Буг",
        "Новий Калинів",
        "Новий Розділ",
        "Новоазовськ",
        "Нововолинськ",
        "Новоград-Волинський",
        "Новогродівка",
        "Новодністровськ",
        "Новодружеськ",
        "Новомиргород",
        "Новомосковськ",
        "Новоселиця",
        "Новоукраїнка",
        "Новояворівськ",
        "Носівка",
        "Обухів",
        "Овруч",
        "Одеса",
        "Олевськ",
        "Олександрівськ",
        "Олександрія",
        "Олешки",
        "Оріхів",
        "Остер ",
        "Острог",
        "Охтирка",
        "Очаків",
        "П'ятихатки",
        "Павлоград",
        "Первомайськ",
        "Первомайський ",
        "Перевальськ",
        "Перемишляни",
        "Перечин",
        "Перещепине",
        "Переяслав",
        "Першотравенськ",
        "Петрово-Красносілля",
        "Пирятин",
        "Південне ",
        "Підгайці",
        "Підгородне",
        "Погребище",
        "Подільськ",
        "Покров",
        "Покровськ",
        "Пологи ",
        "Полонне",
        "Полтава",
        "Помічна",
        "Попасна",
        "Почаїв",
        "Привілля",
        "Прилуки",
        "Приморськ (Україна)",
        "Прип'ять ",
        "Пустомити",
        "Путивль",
        "Рава-Руська",
        "Радехів",
        "Радивилів",
        "Радомишль",
        "Рахів",
        "Рені",
        "Решетилівка",
        "Ржищів",
        "Рівне",
        "Ровеньки",
        "Рогатин",
        "Родинське",
        "Рожище",
        "Роздільна",
        "Ромни",
        "Рубіжне",
        "Рудки",
        "Саки ",
        "Самбір",
        "Сарни",
        "Свалява",
        "Сватове",
        "Світловодськ",
        "Світлодарськ",
        "Святогірськ",
        "Севастополь",
        "Селидове",
        "Семенівка",
        "Середина-Буда",
        "Сєвєродонецьк",
        "Синельникове",
        "Сіверськ",
        "Сімферополь",
        "Скадовськ",
        "Скалат",
        "Сквира",
        "Сколе",
        "Славута",
        "Славутич",
        "Слов'янськ",
        "Сміла",
        "Снігурівка",
        "Сніжне",
        "Сновськ",
        "Снятин",
        "Сокаль",
        "Сокиряни",
        "Соледар",
        "Сорокине",
        "Соснівка ",
        "Старий Крим",
        "Старий Самбір",
        "Старобільськ",
        "Старокостянтинів",
        "Стебник",
        "Сторожинець",
        "Стрий",
        "Судак ",
        "Судова Вишня",
        "Сулимівка (Слов'янськ)",
        "Суми",
        "Суходільськ",
        "Таврійськ",
        "Тальне",
        "Тараща",
        "Татарбунари",
        "Теплодар",
        "Теребовля",
        "Тернівка ",
        "Тернопіль",
        "Тетіїв",
        "Тиврів",
        "Тисмениця",
        "Тлумач",
        "Токмак",
        "Торецьк",
        "Тростянець",
        "Трускавець",
        "Тульчин",
        "Турка",
        "Тячів",
        "Угнів",
        "Ужгород",
        "Узин",
        "Українка ",
        "Українськ",
        "Умань",
        "Устилуг",
        "Фастів",
        "Федорівка",
        "Феодосія",
        "Харків",
        "Харцизьк",
        "Херсон",
        "Хирів",
        "Хмельницький",
        "Хмільник",
        "Ходорів",
        "Хорол ",
        "Хоростків",
        "Хотин",
        "Хрестівка",
        "Христинівка",
        "Хрустальний",
        "Хуст",
        "Часів Яр",
        "Червоноград",
        "Червоносів",
        "Черкаси",
        "Чернівці",
        "Чернігів",
        "Чигирин",
        "Чистякове",
        "Чоп",
        "Чорнобиль",
        "Чорноморськ",
        "Чортків",
        "Чугуїв",
        "Чуднів",
        "Шаргород",
        "Шахтарськ",
        "Шепетівка",
        "Шостка",
        "Шпола",
        "Шумськ",
        "Щастя ",
        "Щолкіне",
        "Южне",
        "Южноукраїнськ",
        "Яворів",
        "Яготин",
        "Ялта",
        "Ямпіль",
        "Яремче",
        "Ясинувата",
    )

    region_names = (
        "Автономна Республіка Крим",
        "Вінницька область",
        "Волинська область",
        "Дніпропетровська область",
        "Донецька область",
        "Житомирська область",
        "Закарпатська область",
        "Запорізька область",
        "Івано-Франківська область",
        "Київська область",
        "Кіровоградська область",
        "Луганська область",
        "Львівська область",
        "Миколаївська область",
        "Одеська область",
        "Полтавська область",
        "Рівненська область",
        "Сумська область",
        "Тернопільська область",
        "Харківська область",
        "Херсонська область",
        "Хмельницька область",
        "Черкаська область",
        "Чернівецька область",
        "Чернігівська область",
    )

    countries = [
        "Австралія",
        "Австрія",
        "Азербайджан",
        "Албанія",
        "Алжир",
        "Ангола",
        "Андорра",
        "Антигуа і Барбуда",
        "Аргентина",
        "Афганістан",
        "Багамські Острови",
        "Бангладеш",
        "Барбадос",
        "Бахрейн",
        "Беліз",
        "Бельгія",
        "Бенін",
        "Білорусь",
        "Болгарія",
        "Болівія",
        "Боснія і Герцеговина",
        "Ботсвана",
        "Бразилія",
        "Бруней",
        "Буркіна-Фасо",
        "Бурунді",
        "Бутан",
        "Вануату",
        "Ватикан",
        "Велика Британія",
        "Венесуела",
        "В'єтнам",
        "Вірменія",
        "Габон",
        "Гаїті",
        "Гаяна",
        "Гамбія",
        "Гана",
        "Гватемала",
        "Гвінея",
        "Гвінея-Бісау",
        "Гондурас",
        "Гренада",
        "Греція",
        "Грузія",
        "Данія",
        "Джибуті",
        "Домініка",
        "Домініканська Республіка",
        "Еквадор",
        "Екваторіальна Гвінея",
        "Еритрея",
        "Естонія",
        "Ефіопія",
        "Єгипет",
        "Ємен",
        "Замбія",
        "Західна Сахара",
        "Зімбабве",
        "Ізраїль",
        "Індія",
        "Індонезія",
        "Ірак",
        "Іран",
        "Ірландія",
        "Ісландія",
        "Іспанія",
        "Італія",
        "Йорданія",
        "Кабо-Верде",
        "Казахстан",
        "Камбоджа",
        "Камерун",
        "Канада",
        "Катар",
        "Кенія",
        "Киргизстан",
        "КНР",
        "Кіпр",
        "Кірибаті",
        "Колумбія",
        "Коморські Острови",
        "Конго",
        "ДР Конго",
        "Південна Корея",
        "Північна Корея",
        "Косово",
        "Коста-Рика",
        "Кот-д'Івуар",
        "Куба",
        "Кувейт",
        "Лаос",
        "Латвія",
        "Лесото",
        "Литва",
        "Ліберія",
        "Ліван",
        "Лівія",
        "Ліхтенштейн",
        "Люксембург",
        "Маврикій",
        "Мавританія",
        "Мадагаскар",
        "Малаві",
        "Малайзія",
        "Малі",
        "Мальдіви",
        "Мальта",
        "Марокко",
        "Маршаллові Острови",
        "Мексика",
        "Федеративні Штати Мікронезії",
        "Мозамбік",
        "Молдова",
        "Монако",
        "Монголія",
        "М'янма",
        "Намібія",
        "Науру",
        "Непал",
        "Нігер",
        "Нігерія",
        "Нідерланди",
        "Нікарагуа",
        "Німеччина",
        "Нова Зеландія",
        "Норвегія",
        "ОАЕ",
        "Оман",
        "Пакистан",
        "Палау",
        "Палестинська держава",
        "Панама",
        "Папуа Нова Гвінея",
        "ПАР",
        "Парагвай",
        "Перу",
        "Південний Судан",
        "Польща",
        "Португалія",
        "Північна Республіка Македонія",
        "Росія",
        "Руанда",
        "Румунія",
        "Сальвадор",
        "Самоа",
        "Сан-Марино",
        "Сан-Томе і Принсіпі",
        "Саудівська Аравія",
        "Свазіленд",
        "Сейшельські Острови",
        "Сенегал",
        "Сент-Вінсент і Гренадини",
        "Сент-Кіттс і Невіс",
        "Сент-Люсія",
        "Сербія",
        "Сінгапур",
        "Сирія",
        "Словаччина",
        "Словенія",
        "Соломонові Острови",
        "Сомалі",
        "Судан",
        "Суринам",
        "Східний Тимор",
        "США",
        "Сьєрра-Леоне",
        "Таджикистан",
        "Таїланд",
        "Тайвань",
        "Танзанія",
        "Того",
        "Тонга",
        "Тринідад і Тобаго",
        "Тувалу",
        "Туніс",
        "Туреччина",
        "Туркменістан",
        "Уганда",
        "Угорщина",
        "Узбекистан",
        "Україна",
        "Уругвай",
        "Фіджі",
        "Філіппіни",
        "Фінляндія",
        "Франція",
        "Хорватія",
        "Центральноафриканська Республіка",
        "Чад",
        "Чехія",
        "Чилі",
        "Чорногорія",
        "Швейцарія",
        "Швеція",
        "Шрі-Ланка",
        "Ямайка",
        "Японія",
    ]
    street_prefixes = [
        "вулиця",
        "набережна",
        "провулок",
        "площа",
        "проспект",
        "узвіз",
        "сквер",
        "парк",
        "шосе",
    ]

    # https://data.gov.ua/en/dataset/__-120/resource/76e2684f-e75c-45e5-a0b2-6d5596ab47f8
    # https://opendata.drohobych-rada.gov.ua/sites/default/files/%D0%90%D0%B4%D1%80%D0%B5%D1%81%D0%B8%20%D0%94%D1%80%D0%BE%D0%B3%D0%BE%D0%B1%D0%B8%D1%87.xlsx
    street_titles = [
        "1-ша Лінія 4-ї ст. Люстдорфської дороги",
        "1-ша Лінія 6-й ст. Люстдорфської дороги",
        "1-ша Лінія Марії Демченко",
        "1-ша Суворовська",
        "1-й Академічний тупик",
        "10-та Лінія 6-й ст. Люстдорфської дороги",
        "10-та Лінія Марії Демченко",
        "10-та Суворовська",
        "11-ша Лінія 6-й ст. Люстдорфської дороги",
        "11-та Лінія Марії Демченко",
        "11-та ст. Великого Фонтану пляж",
        "11-та Суворовська",
        "12-та Лінія 6-й ст. Люстдорфської дороги",
        "12-та ст. Великого Фонтану пляж",
        "12-та Суворовська",
        "13-та Лінія 6-й ст. Люстдорфської дороги",
        "13-та ст. Великого Фонтану пляж",
        "13-та Суворовська",
        "14-та Лінія 6-й ст. Люстдорфської дороги",
        "14-та ст. Великого Фонтану пляж",
        "14-та Суворовська",
        "15-та Лінія 6-й ст. Люстдорфської дороги",
        "15-та ст. Великого Фонтану пляж",
        "15-та Суворовська",
        "16-та Лінія 6-й ст. Люстдорфської дороги",
        "17-та Лінія 6-й ст. Люстдорфської дороги",
        "18-та Лінія 6-й ст. Люстдорфської дороги",
        "19-та Лінія",
        "19-й км Старокиївської дороги",
        "2-га Лінія 4-ї ст. Люстдорфської дороги",
        "2-га Лінія 6-й ст. Люстдорфської дороги",
        "2-га Лінія Марії Демченко",
        "2-га Суворовська",
        "2-й Академічний тупик",
        "21-й км Старокиївської дороги",
        "3-тя Лінія 4-ї ст. Люстдорфської дороги",
        "3-тя Лінія 6-й ст. Люстдорфської дороги",
        "3-тя Лінія Марії Демченко",
        "3-тя Суворовська",
        "4-та Лінія 6-й ст. Люстдорфської дороги",
        "4-та Лінія Марії Демченко",
        "4-та Суворовська",
        "40-річчя оборони Одеси",
        "411-ї батареї",
        "5-та Лінія 6-й ст. Люстдорфської дороги",
        "5-та Лінія Марії Демченко",
        "5-та Суворовська",
        "6-та Лінія 6-й ст. Люстдорфської дороги",
        "6-та Лінія Марії Демченко",
        "6-та Суворовська",
        "7-ма Лінія 6-й ст. Люстдорфської дороги",
        "7-ма Лінія Дачі Ковалевського",
        "7-ма Суворовська",
        "8-ма Лінія 6-й ст. Люстдорфської дороги",
        "8-ма Лінія Дачі Ковалевського",
        "8-ма Суворовська",
        "9-та Лінія Дачі Ковалевського",
        "9-та Суворовська",
        "Абрикосова",
        "Абрикосовий",
        "Авангардна",
        "Авдєєва-Чорноморського",
        "Авіаторів",
        "Авіаційна",
        "Аграрна",
        "Агрономічна",
        "Агрономічний",
        "Адміральський",
        "Аеродромна",
        "Аеродромний 1-й",
        "Аеродромний 2-й",
        "Аеродромний 3-й",
        "Аеропортівська",
        "Аеропортівський 1-й",
        "Аеропортівський 2-й",
        "Аеропортівський 3-й",
        "Аеропортівський 4-й",
        "Аеропортівський 5-й",
        "Аеропортівський 6-й",
        "Азербайджан",
        "Азовський",
        "Академіка Богатського",
        "Академіка Вавилова",
        "Академіка Векслера",
        "Академіка Вільямса",
        "Академіка Вільямса",
        "Академіка Вільямса",
        "Академіка Воробйова",
        "Академіка Гаркавого",
        "Академіка Глушка",
        "Академіка Заболотного",
        "Академіка Корольова",
        "Академіка Панкратової",
        "Академіка Сахарова",
        "Академіка Філатова",
        "Академіка Ясиновського",
        "Академічна",
        "Академічний",
        "Акордна",
        "Алмазна",
        "Альпіністів",
        "Амбулаторна",
        "Амбулаторний",
        "Амбулаторний 1-й",
        "Амбулаторний 2-й",
        "Амундсена, 1-й",
        "Амундсена, 2-й",
        "Амундсена, 3-й",
        "Амурська",
        "Амурський 1-й",
        "Амурський 2-й",
        "Амурський 3-й",
        "Амурський 4-й",
        "Ананьївська",
        "Ангарська",
        "Андреєвського",
        "Андрійця Олега",
        "Андросовський",
        "Анни Ахматової",
        "Аполона Скальковського",
        "Аптекарський",
        "Аркадіївський",
        "Аркадійська",
        "Аркаса Миколи",
        "Армійська",
        "Артезіанська",
        "Артилерійська",
        "Артилерійський 2-й",
        "Архітекторська",
        "Архітектурна",
        "Асєєва Юрія",
        "Асташкіна",
        "Асташкіна",
        "Астрономічна",
        "Астрономічний",
        "Ашгабатська",
        "Бабеля",
        "Багрицького",
        "Базарна",
        "Байдарочний",
        "Байкал тупик",
        "Байкальська",
        "Балківська",
        "Балтська дорога",
        "Балтський 1-й",
        "Балтський 2-й",
        "Балтський 3-й",
        "Балтський 4-й",
        "Балтський 5-й",
        "Балтський 6-й",
        "Балтський 7-й",
        "Балтський 8-й",
        "Банний",
        "Баркасний",
        "Басейна",
        "Басейний 1-й",
        "Басейний 2-й",
        "Басейний 3-й",
        "Басейний 4-й",
        "Басейний 6-й",
        "Бахчева",
        "Баштанна",
        "Безіменний",
        "Берегова",
        "Березова",
        "Березовий",
        "Бернардацці",
        "Бессарабська",
        "Бехтерєва",
        "Бібліотечна",
        "Білоруська",
        "Більшовицький",
        "Біляївська",
        "Бірюкова",
        "Бісквітний",
        "Бітумна",
        "Благовидової",
        "Богдана Хмельницького",
        "Богуна",
        "Бодаревського",
        "Бокаріуса Миколи",
        "Болгарська",
        "Болградська",
        "Бориса Дерев'янка",
        "Бориса Літвака",
        "Бородінська",
        "Ботанічний",
        "Братів Поджіо",
        "Братська",
        "Брестська",
        "Бреуса",
        "Бригадна",
        "Бугаївська",
        "Будівельна",
        "Будівельний",
        "Бузкова",
        "Бузковий",
        "Бузковий 1-й",
        "Бузковий 2-й",
        "Буніна",
        "Бучми",
        "В’ячеслава Чорновола",
        "Валіховський",
        "Ванний",
        "Ванцетті",
        "Ванцетті",
        "Вапняна",
        "Вапняне селище",
        "Вапняний 1-й",
        "Вапняний 2-й",
        "Вапняний 3-й",
        "Варненська",
        "Василя Кандинського",
        "Василя Кандинського 1-й",
        "Василя Кандинського 2-й",
        "Василя Кандинського 3-й",
        "Василя Кандинського 4-й",
        "Василя Кандинського 5-й",
        "Василя Симоненка",
        "Василя Стуса",
        "Васнецова",
        "Васнецова",
        "Ватманський",
        "Велика Арнаутська",
        "Велика Садова",
        "Венгера",
        "Верещагіна",
        "Вернидуба",
        "Верстатобудівна",
        "Вертелецького",
        "Верф’яний",
        "Верхня",
        "Вершинна",
        "Весела",
        "Весняна",
        "Ветеранів праці",
        "Вєтрова",
        "Вєтрогонова",
        "Виїзна",
        "Виноградна",
        "Виноградна 1-ша",
        "Виноградна 2-га",
        "Виноградна 3-тя",
        "Виноградний",
        "Виноградний тупик",
        "Висока",
        "Високий",
        "Висоцького",
        "Виставочна",
        "Вишнева",
        "Вишневий",
        "Відважних",
        "Військовий узвіз",
        "Вільгельма Габсбурга",
        "Віри Інбер",
        "Віри Фігнер",
        "Віри Холодної",
        "Вірського",
        "Вітчизняна",
        "Віцеадмірала Азарова",
        "Віцеадмірала Жукова",
        "Водний",
        "Водопровідна",
        "Водопровідний 1-й",
        "Водопровідний 2-й",
        "Водопровідний 3-й",
        "Вознесенський",
        "Вокзальна",
        "Вокзальний",
        "Волзький",
        "Волна тупик",
        "Волниста",
        "Володимира Вінниченка",
        "Володимира Хавкіна",
        "Володі Дубініна",
        "Волоколамська",
        "Воронезька",
        "Воронцовський",
        "Восьмого березня",
        "Восьмого березня 1-й",
        "Восьмого березня 2-й",
        "Восьмого березня 3-й",
        "Восьмого березня 4-й",
        "Восьмого березня 5-й",
        "Восьмого березня 6-й",
        "Восьмого березня 7-й",
        "Восьмого березня 8-й",
        "Восьмого березня 9-й",
        "Восьмого Березня Лінія 1-ша",
        "Восьмого Березня Лінія 2-га",
        "Восьмого Березня Лінія 3-тя",
        "Восьмого Березня Лінія 4-та",
        "Восьмого Березня Лінія 5-та",
        "Восьмого березня узвіз",
        "Восьмого березня узвіз",
        "Вузький",
        "Вчительська",
        "Вчительської, 2-й",
        "Гаванна",
        "Гагаріна",
        "Гагаріна",
        "Гагаріна",
        "Гагарінське плато",
        "Газова",
        "Газовий",
        "Гаріна",
        "Гаршина",
        "Гаршина",
        "Гастелло",
        "Гвардійська",
        "Гвоздична",
        "Гвоздичний",
        "Гена Іоганна",
        "Генуезька",
        "Геологічна",
        "Георгія Гамова",
        "Геранієва",
        "Героїв Крут",
        "Героїв Небесної Сотні",
        "Героїв оборони Одеси",
        "Героїв прикордонників",
        "Герцена",
        "Гетьманський",
        "Гілельса",
        "Гімназична",
        "Гладкова",
        "Глазунова",
        "Глазунова 1-й",
        "Глазунова 2-й",
        "Глиняна",
        "Глухий міст",
        "Гоголя",
        "Головківська",
        "Головна",
        "Гонтаренко",
        "Горбатий міст",
        "Горбиста",
        "Горизонтальна",
        "Горіхова",
        "Городня",
        "Госпітальний",
        "Градоначальницька",
        "Гранатна",
        "Гранатний",
        "Грецька",
        "Грецька",
        "Грецький",
        "Грибоєдова",
        "Грузинська",
        "Грузовий",
        "Давида Ойстраха",
        "Далекосхідна",
        "Дальницька",
        "Дальницьке",
        "Дальній",
        "Дальня",
        "Данила Крижанівського",
        "Данькевича",
        "Дача Ковалевського",
        "Дачна",
        "Дачний",
        "Дачний 1-й",
        "Дачний 2-й",
        "Дворянська",
        "Дев’ята",
        "Деволанівська",
        "Деволанівський узвіз",
        "Дежньова",
        "Декабристів",
        "Дельфін",
        "Дем’янова",
        "Демократична",
        "Депутатський",
        "Деревообробна",
        "Деревообробний 1-й",
        "Деревообробний 2-й",
        "Деревообробний 3-й",
        "Деревообробний 4-й",
        "Деревообробний 5-й",
        "Державіна",
        "Державіна",
        "Дерибасівська",
        "Десантний",
        "Десята",
        "Десятого квітня",
        "Десятого квітня",
        "Джерельна",
        "Джутовий",
        "Дзвінка",
        "Дігтярна",
        "Дідріхсона",
        "Діхтієвського Віктора",
        "Дмитрія Донського",
        "Дмитрія Донського",
        "Дніпровська",
        "Дніпропетровська дорога",
        "Дністровська",
        "Добровольського",
        "Довга",
        "Довженка",
        "Докова",
        "Долинська",
        "Донецька",
        "Донцова Дмитра",
        "Дорбуду",
        "Дорожня",
        "Достоєвського",
        "Достоєвського",
        "Друга",
        "Дружний",
        "Дубова",
        "Дубовий гай",
        "Дукова",
        "Думська",
        "Дунаєва",
        "Дунаєвського",
        "Дунаєвського 1-й",
        "Дунаєвського 2-й",
        "Дунаєвського 3-й",
        "Дунайська",
        "Дюківська",
        "Дюківський сад",
        "Економічний",
        "Елеваторна",
        "Естафетна",
        "Естонська",
        "Естонський",
        "Єврейська",
        "Єлисаветградський",
        "Єліна",
        "Єлісаветинська",
        "Єнісейська",
        "Єрмака",
        "Єфімова",
        "Жаботинського",
        "Жасминна",
        "Жевахова",
        "Желябова",
        "Житкова",
        "Житомирська",
        "Житомирський 3-й",
        "Житомирський 4-й",
        "Жоліо-Кюрі",
        "Жолкова Бориса",
        "Жуковського",
        "Заводська",
        "Заводська 1-ша",
        "Заводська 2-га",
        "Заводська 3-тя",
        "Заводська 4-та",
        "Заводська 5-та",
        "Задніпровського",
        "Закарпатська",
        "Залізнична",
        "Залізничний 3-й",
        "Заньковецької",
        "Запорізька",
        "Зарічна",
        "Західний 1-й",
        "Західний 2-й",
        "Західний 3-й",
        "Західний 4-й",
        "Зелена",
        "Зелена балка",
        "Зелена гірка",
        "Зелений",
        "Зернова",
        "Злакова",
        "Змієнка Всеволода",
        "Зої Космодем’янської",
        "Золотий берег",
        "Зоопаркова",
        "Зоринська",
        "Зоряна",
        "Івана Вазова",
        "Івана Микитенка",
        "Івана Франка",
        "Івана Франка",
        "Іванівська",
        "Іванівський переїзд",
        "Іванова",
        "Іванова 1-й",
        "Іванова 2-й",
        "Іванова Ігоря",
        "Івасюка Володимира",
        "Івахненка Петра",
        "Ільфа і Петрова",
        "ім. С.Ю. Вітте",
        "ім. К.Г. Паустовського",
        "Інглезі",
        "Індійська",
        "Інтернаціональний",
        "Іподромний",
        "Іспанська",
        "Іспанський",
        "Історія Футболу",
        "Італійський",
        "Іцхака Рабіна",
        "Йосипа Тимченка",
        "Кавказька",
        "Кавказький",
        "Кавунова",
        "Казанська",
        "Казковий",
        "Калинова",
        "Калнишевського Петра",
        "Каманіна",
        "Каманіна",
        "Камишова",
        "Канатна",
        "Канатний",
        "Капітана Гаврікова",
        "Капітана Кузнецова",
        "Кар’єрна",
        "Карантинна",
        "Карантинний узвіз",
        "Каретний",
        "Каркашадзе",
        "Карпатська",
        "Картамишевська",
        "Картамишевський",
        "Касима Юрія",
        "Каспійський",
        "Катаєва",
        "Катерининська",
        "Катерининська",
        "Каховський",
        "Качалова",
        "Каштанова",
        "Квіткова",
        "Квітковий",
        "Квітнева",
        "Кедровий",
        "Керченська",
        "Керченський 1-й",
        "Керченський 2-й",
        "Керченський 3-й",
        "Кибальчича",
        "Київське",
        "Килимовий",
        "Кипарисний 1-й",
        "Кипарисний 2-й",
        "Кипарисний 3-й",
        "Китобійна",
        "Китобійний 1-й",
        "Китобійний 2-й",
        "Кифоренка Бориса",
        "Кишинівська",
        "Кільовий",
        "Кільцева",
        "Кінна",
        "Кісельова Ігоря",
        "Кладовищенська",
        "Кладовищний",
        "Кленова",
        "Клинова",
        "Ключовий 1-й",
        "Ключовий 2-й",
        "Книжковий",
        "Князівська",
        "Князівський",
        "Коблевська",
        "Ковалевського",
        "Ковалевського узвіз",
        "Ковиловий",
        "Колективний",
        "Колекційна",
        "Колонічна",
        "Колонтаївська",
        "Комітетська",
        "Компасний",
        "Композитора Глинки",
        "Композитора Глинки",
        "Композитора Ніщинського",
        "Кондрашина",
        "Кондренка",
        "Конструкторський",
        "Контрадмірала Луніна",
        "Кордонна",
        "Кордонний",
        "Корнюшина",
        "Короленка",
        "Короткий",
        "Косвена",
        "Косий",
        "Космонавта Комарова",
        "Космонавтів",
        "Косовська",
        "Косовський",
        "Костанді",
        "Костанді 2-й",
        "Костанді 3-й",
        "Кострова",
        "Косяченка Олексія",
        "Котляревського",
        "Коцебу міст",
        "Коцюбинського",
        "Кощового отамана",
        "Крайня",
        "Красива",
        "Красна",
        "Красний",
        "Красних Зорь",
        "Краснова",
        "Красносільська",
        "Краснослобідська",
        "Краснослобідський",
        "Кредитний",
        "Кренкеля",
        "Кривобалківська",
        "Крижанівський",
        "Крилова",
        "Кримська",
        "Кримський",
        "Кристаловського",
        "Кришталева",
        "Кришталевий",
        "Кропивницького",
        "Кругова",
        "Круговий",
        "Крутоярська",
        "Крушельницької",
        "Кузнечна",
        "Куликове поле",
        "Куликовський 2-й",
        "Куниці",
        "Купріна",
        "Курганська",
        "Курортний",
        "Курортний 6-й",
        "Курська",
        "Кустанайська",
        "Кустанайський 2-й",
        "Кутова",
        "Кутузова",
        "Куяльницький міст",
        "Лавкова",
        "Лазурний 1-й",
        "Лазурний 2-й",
        "Лазурний 3-й",
        "Лазурний 4-й",
        "Лазурний 5-й",
        "Лазурний 6-й",
        "Ламаний",
        "Ланжерон",
        "Ланжеронівська",
        "Ланжеронівський узвіз",
        "Латвійський узвіз",
        "Левадна",
        "Леваневського",
        "Леваневського",
        "Леваневського",
        "Леваневського тупик",
        "Левітана",
        "Левкоєва",
        "Лейтенанта Шмідта",
        "Леонова",
        "Леонтовича",
        "Лермонтовський",
        "Лермонтовський 2-й",
        "Лесі Українки",
        "Лесі Українки",
        "Леха Качинського",
        "Лиманна",
        "Лиманний",
        "Лиманчик 2-й Лінія 1-ша",
        "Лиманчик 2-й Лінія 11-та",
        "Лиманчик 2-й Лінія 2-га",
        "Лиманчик 2-й Лінія 3-тя",
        "Лиманчик 2-й Лінія 4-та",
        "Лиманчик 2-й Лінія 5-та",
        "Лиманчик 2-й Лінія 8-ма",
        "Лиманчик 2-й Лінія 9-та",
        "Лип Івана та Юрія",
        "Липнева",
        "Листяна",
        "Листяний",
        "Литовська",
        "Ліверпульський",
        "Лідерсівський",
        "Лізи Чайкіної",
        "Лінійна",
        "Лінійний",
        "Лїнія 1-ша",
        "Лінія 22-га",
        "Лінія 23-тя",
        "Лінія 24-та",
        "Лінія 25-та",
        "Лінія 26-та",
        "Лінія 27-ма",
        "Лінія 28-ма",
        "Лінія 29-та",
        "Лінія 30-та",
        "Лінія 31-ша",
        "Лінія 32-га",
        "Лінія 33-тя",
        "Лінія 35-та",
        "Лінія 36-та",
        "Лінія 37-ма",
        "Лінія 38-ма",
        "Лінія 39-та",
        "Лінія 40-ва",
        "Лінія 41-ша",
        "Лінія 42-га",
        "Лінія 43-тя",
        "Лінія 44-та",
        "Лінія 45-та",
        "Лінія 46-та",
        "Лінія 47-ма",
        "Лінія 48-ма",
        "Лінія 49-та",
        "Лінія 50-та",
        "Лінія 51-ша",
        "Лінія 52-га",
        "Лінія 53-тя",
        "Лінія 54-та",
        "Лінія 55-та",
        "Літакова",
        "Літературна",
        "Літній",
        "Літня",
        "Лобачевського",
        "Лодочний",
        "Локальна",
        "Локомотивна",
        "Локомотивний 1-й",
        "Локомотивний 2-й",
        "Локомотивний 3-й",
        "Локомотивний 4-й",
        "Ломоносова",
        "Луганська",
        "Лугова",
        "Лузанівка",
        "Лузанівка",
        "Лузанівська",
        "Лузанівський",
        "Лузанівський 1-й",
        "Лузанівський 2-й",
        "Лунний",
        "Луценка Івана",
        "Луцька",
        "Лучиста",
        "Льва Симиренка",
        "Льва Толстого",
        "Льва Толстого",
        "Львівська",
        "Львівський",
        "Любашівський",
        "Людмили Гінзбург",
        "Люстдорфська дорога",
        "Люстдорфський",
        "Лютеранський",
        "Лютнева",
        "Лютневий 1-й",
        "Лютневий 2-й",
        "Лядова",
        "Ляпідевського",
        "Ляпунова",
        "М’ясоєдовська",
        "Магістральна",
        "Магнітогорська",
        "Магнітогорський",
        "Магнітогорський 2-й",
        "Магнітогорський 3-й",
        "Мазараті",
        "Мазепи Івана",
        "Майстерний",
        "Макаренка",
        "Макарова",
        "Макова",
        "Маковий",
        "Мала",
        "Мала Арнаутська",
        "Мала Садова",
        "Маланова",
        "Малиновський",
        "Маловського",
        "Манежна",
        "Манежний",
        "Маразліївська",
        "Марата",
        "Марата 1-й",
        "Марата 2-й",
        "Маринеско узвіз",
        "Мариністів",
        "Марії Демченко",
        "Марії Демченко",
        "Марії Демченко 2-й",
        "Маріїнська",
        "Марсельська",
        "Маршала Бабаджаняна",
        "Маршала Говорова",
        "Маршала Малиновського",
        "Маршрутна",
        "Матеріальний 1-й",
        "Матроська Слобідка",
        "Матроська Слобідка",
        "Матроський узвіз",
        "Матюшенка",
        "Махачкалинська",
        "Мацієвської",
        "Мацієвської узвіз",
        "Машинобудівний",
        "Маяковського",
        "Маячний",
        "Мелітопольська",
        "Мельницька",
        "Металістів",
        "Метрополітенівський",
        "Мечникова",
        "Мечникова",
        "Мигдальна",
        "Миколаївська дорога",
        "Миколаївський 1-й",
        "Миколи Бажана",
        "Миколи Боровського",
        "Миколи Вороного",
        "Миколи Гефта",
        "Миколи Гумільова",
        "Миколи Огренича",
        "Миколи Плигуна",
        "Мирна",
        "Митна",
        "Митракова",
        "Михайла Божія",
        "Михайла Грушевського",
        "Михайла Жванецького",
        "Михайлівська",
        "Михайлівська",
        "Мінська",
        "Місячна",
        "Місячна",
        "Міхновського Миколи",
        "Міцкевича",
        "Мічманський 1-й",
        "Мічманський 2-й",
        "Мічуріна",
        "Мічуріна",
        "Мічуріна",
        "Могилівська",
        "Молоді",
        "Молодіжна",
        "Молодіжна",
        "Молодогвардійська",
        "Молокова",
        "Монастирський",
        "Монгольська",
        "Морехідна",
        "Морехідний",
        "Морська",
        "Морський",
        "Морський 2-й",
        "Москвіна",
        "Москеті",
        "Мостовий",
        "Моторна",
        "Моторний 1-й",
        "Моторний 2-й",
        "Моторний 3-й",
        "Мукачівський",
        "Мусіна",
        "Набережна",
        "Навігаційний",
        "Надіїі",
        "Наклонна",
        "Наливна",
        "Нафтовиків",
        "Нафтовиків",
        "Нафтовиків 1-й",
        "Нафтовиків 2-й",
        "Нафтовиків 3-й",
        "Нахімова",
        "Наявний 1-й",
        "Наявний 2-й",
        "Наявний 3-й",
        "Наявний 4-й",
        "Наявний 5-й",
        "Наявний 6-й",
        "Наявний 7-й",
        "Небесної Сотні",
        "Недєліна",
        "Нежданової",
        "Нежданової",
        "Незалежності",
        "Некрасова",
        "Немировича-Данченка",
        "Неплія",
        "Нерубайська",
        "Нескучна",
        "Нечипуренка",
        "Ніжинська",
        "Нікітіна",
        "Нова",
        "Нова",
        "Нова",
        "Новаторів",
        "Новгородська",
        "Новгородський",
        "Новгородський 2-й",
        "Новий",
        "Новиков міст",
        "Новикова",
        "Новікова 2-га",
        "Новобазарний",
        "Новоберегова",
        "Новомосковська дорога",
        "Новоселів",
        "Новоселів",
        "Новосельского",
        "Новоукраїнський",
        "Новощіпний ряд",
        "Об'їзна дорога",
        "Обільна",
        "Обільний 1-й",
        "Обільний 2-й",
        "Оборони Ленінграду",
        "Обривиста",
        "Обсерваторний",
        "Овідіопольська",
        "Овідіопольська дорога",
        "Овідіопольська дуга",
        "Овідіопольський",
        "Одарія",
        "Одеса-Головна",
        "Одеська",
        "Одеський",
        "Озерна",
        "Окружна",
        "Олександра Блока",
        "Олександра Вронського",
        "Олександра Кутузакія",
        "Олександра Матросова",
        "Олександра Невського",
        "Олександра Невського 1-й",
        "Олександра Невського 2-й",
        "Олександра Невського 3-й",
        "Олександра Невського 4-й",
        "Олександра Невського 5-й",
        "Олександра Стурдзи",
        "Олександра Тимошенка",
        "Олександрівський",
        "Олексіївська",
        "Ольгіївська",
        "Ольгіївський узвіз",
        "Омська",
        "Онезька",
        "Онілової",
        "Оранжерейний",
        "Орликова",
        "Орловська",
        "Орловський",
        "Осипова",
        "Осіння",
        "Остапа Вишні",
        "Отамана Головатого",
        "Отамана Чепіги",
        "Отрада",
        "Отрадна",
        "Офіцерська",
        "Офіцерський 1-й",
        "Офіцерський 2-й",
        "П'ята",
        "Пiвденний",
        "Павла Кравцова",
        "Павла Шклярука",
        "Павлова",
        "Павлодарська",
        "Палія Семена",
        "Палубна",
        "Палубний",
        "Панаса Мирного",
        "Пантелеймонівська",
        "Панченка",
        "Парашутний",
        "Паркова",
        "Парковий",
        "Партизанська",
        "Партизанської Слави",
        "Парусна",
        "Пассіонарії",
        "Пастера",
        "Патріотична",
        "Паустовського",
        "Педагогічна",
        "Педагогічний",
        "Пейзажна",
        "Перемоги",
        "Перемоги",
        "Перепечка",
        "Пересипський міст",
        "Пересипська 1-ша",
        "Пересипська 10-та",
        "Пересипська 2-га",
        "Пересипська 3-тя",
        "Пересипська 4-та",
        "Пересипська 5-та",
        "Пересипська 6-та",
        "Пересипська 7-ма",
        "Пересипська 8-ма",
        "Пересипська 9-та",
        "Пересипський 1-й",
        "Пересипський 2-й",
        "Пересипський узвіз",
        "Перлинна",
        "Перша",
        "Перший Кришталевий",
        "Першотравневий 1-й",
        "Першотравневий 2-й",
        "Першотравневий 3-й",
        "Пестеля",
        "Пестеля",
        "Петра Лещенка",
        "Петрашевського",
        "Пироговська",
        "Пироговський",
        "Писарева",
        "Південна",
        "Південна дорога",
        "Південно-Санаторний",
        "Північна",
        "Північний",
        "Підйомний",
        "Пілотна",
        "Пісочна",
        "Пісочний",
        "Пішонівська",
        "Планетна",
        "Платанова",
        "Плієва",
        "Пляжна",
        "Побратимів",
        "Подільська",
        "Поїзна",
        "Покровський",
        "Ползунова 1-й",
        "Ползунова 2-й",
        "Політкаторжан",
        "Політкаторжан",
        "Полтавська",
        "Полуничний",
        "Польова",
        "Польова",
        "Польовий 1-й",
        "Польська",
        "Польський узвіз",
        "Поперечний",
        "Посівна",
        "Посмітного",
        "Поштова",
        "Поштовий",
        "Преображенська",
        "Пресича Олександра",
        "Пржевальського",
        "Прибережний",
        "Привозна",
        "Привокзальна",
        "Привокзальний",
        "Приморська",
        "Приморський",
        "Приморський",
        "Прирейкова",
        "Проектна 1-ша",
        "Проектна 2-га",
        "Проектний 5-й",
        "Проектований 3-й",
        "Проектований 4-й",
        "Проїзний 1-й",
        "Проїзний 2-й",
        "Проїзний 3-й",
        "Прокатна",
        "Пролетарський 3-й",
        "Промислова",
        "Промисловий",
        "Прорізна",
        "Прорізний",
        "Просвіти",
        "Просьолочна",
        "Проточний",
        "Професора Каришковського",
        "Професора Коровицького",
        "Профспілкова",
        "Прохоровська",
        "Прохоровський",
        "Прохоровський",
        "Проценка",
        "Псковська",
        "Псковський",
        "Пугачова",
        "Путьова",
        "Пушкінська",
        "ПшеничнаРадищева",
        "Радіальна",
        "Радіальний",
        "Радіо",
        "Радісна",
        "Радісний",
        "Разумовська",
        "Раїси Сергієнко",
        "Райдужна",
        "Ракетна",
        "Рачкова",
        "Регенсбург",
        "Рекордний",
        "Рельєфна",
        "Ремонтна",
        "Республіканська",
        "Рєпіна",
        "Рибальська",
        "Рибальська балка",
        "Рибальський",
        "Ризька",
        "Рівний",
        "Рівності",
        "Рівності",
        "Рівності 1-й",
        "Рівності 2-й",
        "Різдвяна",
        "Різницький",
        "Різовська",
        "Рішельєвська",
        "Рожева",
        "Роздольна",
        "Розкидайлівська",
        "Розумовський 1-й",
        "Розумовський 2-й",
        "Романа Кармена",
        "Романтиків",
        "Романтичний",
        "Ромашкова",
        "Ростовська",
        "Рощева",
        "Рульовий",
        "Сабанєєв",
        "Сабанський",
        "Савицький",
        "Савранська",
        "Садиковська",
        "Садова",
        "Садова 1-ша",
        "Садова 2-га",
        "Садова 3-тя",
        "Садова 4-та",
        "Садова 5-та",
        "Садова 6-та",
        "Садовий",
        "Садовського",
        "Саксаганського",
        "Салтикова-Щедріна",
        "Самарська",
        "Самодіяльна",
        "Санаторний",
        "Санітарна",
        "Сашка Хорошенка",
        "Світанку",
        "Світла",
        "Світлий",
        "Свободи",
        "Святослава Ріхтера",
        "Севастопольський",
        "Сегедська",
        "Селекційна",
        "Селекційний",
        "Селищна",
        "Селищний",
        "Семафорний",
        "Семінарська",
        "Сергія Ейзенштейна",
        "Сергія Уточкіна",
        "Сергія Ядова",
        "Серединський",
        "Середньофонтанська",
        "Середньофонтанський",
        "Середня",
        "Сєрова",
        "Сєрогодського",
        "Сєченова",
        "Сибірська",
        "Сирітський",
        "Сирітський 2-й",
        "Сільська",
        "Сільськогосподарський",
        "Сімферопольська",
        "Сінна",
        "Скворцова",
        "Скидановська",
        "Скидановський узвіз",
        "Складська",
        "Скляна",
        "Скляний 1-й",
        "Скляний 2-й",
        "Скляний 3-й",
        "Скрипковий",
        "Слави",
        "Слави",
        "Слєпньова",
        "Сливова",
        "Слобідська",
        "Слобідський узвіз",
        "Слов'янська",
        "Сміливий",
        "Смоленська",
        "Собінова",
        "Соборна",
        "Совіньйонівський",
        "Солонцюватий",
        "Солонцюватий 1-й",
        "Соляна",
        "Соляний 1-й",
        "Соляний 2-й",
        "Соляний 3-й",
        "Соляний 4-й",
        "Соляний 5-й",
        "Соляний 6-й",
        "Соляний 7-й",
        "Сонячна",
        "Сонячна",
        "Сортувальна 1-ша",
        "Сортувальна 2-га",
        "Сосюри",
        "Софії Перовської",
        "Софіївська",
        "Соціальна",
        "Спартаківська",
        "Спартаківський",
        "Спаський",
        "Спиридонівська",
        "Спортивна",
        "Спортивний",
        "Сталепрокатна",
        "Стамбульський",
        "Станційна 1-ша",
        "Станційна 2-га",
        "Станційна 3-тя",
        "Старицького",
        "Старобазарний",
        "Старокінний",
        "Старопортофранківська",
        "Старорізнична",
        "Старосінна",
        "Стахановський 4-й",
        "Стельмаха",
        "Степана Олійника узвіз",
        "Степана Разіна",
        "Степна",
        "Степний 2-й",
        "Степова",
        "Стеценко",
        "Стєклова",
        "Стовпова",
        "Стороженка Олега",
        "Стражеска Миколи",
        "Строганов міст",
        "Строганова",
        "Студена",
        "Студентський",
        "Студентський 1-й",
        "Студентський 2-й",
        "Студентський 3-й",
        "Студентський 4-й",
        "Студентський 5-й",
        "Суворовська",
        "Суднобудівна",
        "Суднобудівний",
        "Сумська",
        "Сумський",
        "Суперфосфатна",
        "Супутників",
        "Сурикова",
        "Сурикова 1-й",
        "Сурикова 2-й",
        "Сухолиманна",
        "Східний",
        "Східний 1-й",
        "Східний 2-й",
        "Східчастий 3-й",
        "Східчастий 4-й",
        "Східчастий",
        "Сьома",
        "Таїрова",
        "Таїрова",
        "Танкерна",
        "Танкістів",
        "Танфільєва",
        "Тарутинська",
        "Творча",
        "Текстильників",
        "Теліги Олени",
        "Тепла",
        "Теплична",
        "Тепличний 1-й",
        "Тепличний 2-й",
        "Терещенка Володимира",
        "Тетяни Тесс",
        "Технічний",
        "Тещин міст",
        "Тимірязєва",
        "Тимірязєва",
        "Тимірязєва 3-й",
        "Тимірязєва 4-й",
        "Тимірязєва 5-й",
        "Тираспольська",
        "Тираспольська",
        "Тираспольське",
        "Тиха",
        "Тіниста",
        "Тінистий",
        "Тінистий",
        "Ткачова",
        "Товарний",
        "Толбухіна",
        "Толбухіна",
        "Толбухіна",
        "Тополина",
        "Тополиний",
        "Топольського",
        "Торгова",
        "Травневий",
        "Травневий 1-й",
        "Травневий 2-й",
        "Травневий 3-й",
        "Травневий 4-й",
        "Травневий 5-й",
        "Тракторна",
        "Трамвайна",
        "Трамвайний",
        "Транспортна",
        "Траса Здоров'я",
        "Троїцька",
        "Троїцького Миколи",
        "Тролейбусна",
        "Трояндова",
        "Трудова",
        "Трудовий",
        "Трудових резервів",
        "Тульська",
        "Тульська",
        "Тупиковий",
        "Тупиковий 1-й",
        "Тупиковий 2-й",
        "Тургенєва",
        "Туристська",
        "Туристський",
        "Тюльпанний",
        "Угорська",
        "Удільний",
        "Українська",
        "Український 1-й",
        "Український 2-й",
        "Український 3-й",
        "Улітіна",
        "Уманська",
        "Умова",
        "Університетський",
        "Уральська",
        "Уральський",
        "Урожайна",
        "Усатівська",
        "Успенська",
        "Успенський",
        "Утьосова",
        "Училищна",
        "Учительський",
        "Учнівська",
        "Ушакова",
        "Ушинського",
        "Уютна",
        "Ф. Пішеніна",
        "Фабрична",
        "Фестивальний",
        "Флотська",
        "Флотський 1-й",
        "Флотський 2-й",
        "Флотський 3-й",
        "Флотський 4-й",
        "Фонтанська дорога",
        "Французький",
        "Фруктовий",
        "Хаджибейська дорога",
        "Хаджибейський 1-й",
        "Хаджибейський 2-й",
        "Хаджибейський 3-й",
        "Харківська",
        "Хвойний",
        "Херсонський",
        "Херсонський сквер",
        "Хімічна",
        "Хімічний 1-й",
        "Хімічний 2-й",
        "Хімічний 3-й",
        "Хлібна гавань",
        "Художника Федорова",
        "Хутірська",
        "Хутірський",
        "Цебриківська",
        "Цегельнозаводська",
        "Цегляна",
        "Цементний",
        "Центральна",
        "Центральний аеропорт",
        "Церковна",
        "Цимлянська",
        "Ціолковського",
        "Чайковського",
        "Чапаєва",
        "Чапаєва",
        "Чапаєва",
        "Чапаєва 2-й",
        "Черепанових",
        "Черепанових 2-й",
        "Черешнева",
        "Черкаська",
        "Чернишевського",
        "Чернівецький",
        "Чернігівська",
        "Чернігівський",
        "Черняховського",
        "Четверта",
        "Чехова",
        "Чорноморка",
        "Чорноморська",
        "Чорноморський",
        "Чорноморський",
        "Чорноморський 1-й",
        "Чорноморський 10-й",
        "Чорноморський 11-й",
        "Чорноморський 12-й",
        "Чорноморський 13-й",
        "Чорноморський 2-й",
        "Чорноморський 3-й",
        "Чорноморський 4-й",
        "Чорноморський 5-й",
        "Чорноморський 6-й",
        "Чорноморський 7-й",
        "Чорноморський 8-й",
        "Чорноморський 9-й",
        "Чорноморського козацтва",
        "Чубаївська",
        "Чубаївський",
        "Чумацька",
        "Шампанський",
        "Шахтинський",
        "Шебелінський",
        "Шевченка",
        "Шевченка парк",
        "Шептицікого",
        "Шефська",
        "Шилова",
        "Ширшова",
        "Ширяївський",
        "Шишкіна",
        "Шишкіна",
        "Шишкіна 1-й",
        "Шишкіна 2-й",
        "Шишкіна 3-й",
        "Шкільний",
        "Шкільний аеродром",
        "Шкіперська",
        "Шкодова гора",
        "Шовкуненка",
        "Шовкуненка",
        "Шолохова",
        "Шоста",
        "Шостої 1-й",
        "Шостої 2-й",
        "Шота Руставелі",
        "Штабний",
        "Штильова",
        "Штурвальна",
        "Штурманська",
        "Шухевича Романа",
        "Щіпний",
        "Щоглова",
        "Щоголєва",
        "Щукіна",
        "Ювілейний 1-й",
        "Ювілейний 2-й",
        "Юннатів",
        "Юннатів 1-й",
        "Юннатів 2-й",
        "Юннатів 3-й",
        "Юрженка Олександра",
        "Юрія Олеші",
        "Юрія Яновського",
        "Юхима Геллера",
        "Юхима Фесенка",
        "Яблочкіної",
        "Яблунева",
        "Яблучна",
        "Якірний",
        "Якутська",
        "Ялинкова",
        "Ялинковий",
        "Ямська",
        "Ямчитського",
        "Ярморочна площа",
        "Яружний",
        "Ясна",
        "Яхненка Семена",
        "Яші Гордієнка",
    ]

    def city_prefix(self) -> str:
        return self.random_element(self.city_prefixes)

    def city_name(self) -> str:
        return self.random_element(self.city_names)

    def postcode(self) -> str:
        """The code consists of five digits (01000-99999)"""
        return f"{self.generator.random.randrange(1000, 99999):05}"

    def street_prefix(self) -> str:
        return self.random_element(self.street_prefixes)

    def street_name(self) -> str:
        return self.random_element(self.street_titles)

    def street_title(self) -> str:
        prefix = self.street_prefix()
        street = self.street_name()
        return prefix + " " + street

    def region(self) -> str:
        return self.random_element(self.region_names)



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{company_prefix}}{{last_name}}{{company_category}}",
        "{{last_name}}{{company_category}}{{company_prefix}}",
    )

    company_prefixes = ("株式会社", "有限会社", "合同会社")
    company_categories = (
        "水産",
        "農林",
        "鉱業",
        "建設",
        "食品",
        "印刷",
        "電気",
        "ガス",
        "情報",
        "通信",
        "運輸",
        "銀行",
        "保険",
    )

    def company_prefix(self) -> str:
        return self.random_element(self.company_prefixes)

    def company_category(self) -> str:
        return self.random_element(self.company_categories)



def test_unique_clears(testdir):
    """Successive uses of the `faker` pytest fixture have the
    generated unique values cleared between functions."""

    testdir.makepyfile(
        """
        import pytest
        from faker.exceptions import UniquenessException

        NUM_SAMPLES = 100

        def test_fully_exhaust_unique_booleans(faker):
            _dummy = [faker.boolean() for _ in range(NUM_SAMPLES)]

            faker.unique.boolean()
            faker.unique.boolean()
            with pytest.raises(UniquenessException):
                faker.unique.boolean()
            _dummy = [faker.boolean() for _ in range(NUM_SAMPLES)]

        def test_do_not_exhaust_booleans(faker):
            faker.unique.boolean()

        def test_fully_exhaust_unique_booleans_again(faker):
            _dummy = [faker.boolean() for _ in range(NUM_SAMPLES)]

            faker.unique.boolean()
            faker.unique.boolean()
            with pytest.raises(UniquenessException):
                faker.unique.boolean()
            _dummy = [faker.boolean() for _ in range(NUM_SAMPLES)]
        """,
    )

    result = testdir.runpytest()

    result.assert_outcomes(passed=3)



import re
import unittest

from decimal import Decimal

from faker import Faker
from faker.providers.geo.pt_PT import Provider as PtPtProvider


class TestGlobal(unittest.TestCase):
    """Tests geographic locations regardless of locale"""

    def setUp(self):
        self.fake = Faker()  # No locale specified, gets global for this provider
        Faker.seed(0)

    def test_local_latlng(self):
        loc = self.fake.local_latlng(country_code="US")
        assert isinstance(loc, tuple)
        assert len(loc) == 5
        assert Decimal(loc[0])
        assert Decimal(loc[1])

        loc_short = self.fake.local_latlng(country_code="US", coords_only=True)
        assert len(loc_short) == 2
        assert Decimal(loc_short[0])
        assert Decimal(loc_short[1])


class TestEnUS(unittest.TestCase):
    """Tests geographic locations in the en_US locale"""

    def setUp(self):
        self.fake = Faker("en_US")
        Faker.seed(0)

    def test_latitude(self):
        lat = self.fake.latitude()
        assert isinstance(lat, Decimal)

    def test_longitude(self):
        long = self.fake.longitude()
        assert isinstance(long, Decimal)

    def test_latlng(self):
        loc = self.fake.latlng()
        assert isinstance(loc, tuple)
        assert len(loc) == 2
        assert isinstance(loc[0], Decimal)
        assert isinstance(loc[1], Decimal)

    def test_coordinate(self):
        loc = self.fake.coordinate()
        assert isinstance(loc, Decimal)

    def test_coordinate_centered(self):
        loc = self.fake.coordinate(center=23)
        assert round(loc) == 23

    def test_coordinate_rounded(self):
        loc = self.fake.coordinate(center=23, radius=3)
        assert 20 <= round(loc) <= 26

    def test_location_on_land(self):
        loc = self.fake.location_on_land()
        assert isinstance(loc, tuple)
        assert len(loc) == 5
        assert Decimal(loc[0])  # Should be able to cast first two elements of tuple to Decimal
        assert Decimal(loc[1])
        assert isinstance(loc[2], str)  # Place is a string
        assert isinstance(loc[3], str)  # Country code is a string
        assert len(loc[3]) == 2  # Country code is two letters
        assert isinstance(loc[4], str)  # Timezone is a string

    def test_location_on_land_coords_only(self):
        loc = self.fake.location_on_land(coords_only=True)
        assert isinstance(loc, tuple)
        assert len(loc) == 2
        assert Decimal(loc[0])  # Should be able to cast first two elements of tuple to Decimal
        assert Decimal(loc[1])


class TestSkSk(unittest.TestCase):
    def setUp(self):
        self.fake = Faker("sk_SK")
        Faker.seed(0)

    def test_location_on_land(self):
        loc = self.fake.location_on_land()
        assert isinstance(loc, tuple)
        assert len(loc) == 5
        assert Decimal(loc[0])  # Should be able to cast first two elements of tuple to Decimal
        assert Decimal(loc[1])
        assert isinstance(loc[2], str)  # Place is a string
        assert isinstance(loc[3], str)  # Country code is a string
        assert len(loc[3]) == 2  # Country code is two letters
        assert isinstance(loc[4], str)  # Timezone is a string


class TestCsCz(unittest.TestCase):
    def setUp(self):
        self.fake = Faker("cs_CZ")
        Faker.seed(0)

    def test_location_on_land(self):
        loc = self.fake.location_on_land()
        assert isinstance(loc, tuple)
        assert len(loc) == 5
        assert Decimal(loc[0])  # Should be able to cast first two elements of tuple to Decimal
        assert Decimal(loc[1])
        assert isinstance(loc[2], str)  # Place is a string
        assert isinstance(loc[3], str)  # Country code is a string
        assert len(loc[3]) == 2  # Country code is two letters
        assert isinstance(loc[4], str)  # Timezone is a string


class TestDeAT(unittest.TestCase):
    """Tests in addresses in the de_AT locale"""

    def setUp(self):
        self.fake = Faker("de_AT")
        Faker.seed(0)

    def test_local_latitude(self):
        local_latitude = self.fake.local_latitude()
        assert re.match(r"4[6-8]\.\d+", str(local_latitude))

    def test_local_longitude(self):
        local_longitude = self.fake.local_longitude()
        assert re.match(r"1[1-5]\.\d+", str(local_longitude))


class TestPtPT(unittest.TestCase):
    def setUp(self):
        self.fake = Faker("pt_PT")
        Faker.seed(0)

    def test_nationality(self):
        nationality = self.fake.nationality()
        assert isinstance(nationality, str)
        assert nationality in PtPtProvider.nationalities


class TestTrTr(TestEnUS):
    def setUp(self):
        self.fake = Faker("tr_TR")
        Faker.seed(0)


class TestEnIe(TestEnUS):
    def setUp(self):
        self.fake = Faker("en_IE")
        Faker.seed(0)



from .. import Provider as InternetProvider


class Provider(InternetProvider):
    free_email_domains = (
        "hotmail.com",
        "gmail.com",
        "yahoo.com",
        "yandex.com",
        "yaani.com",
        "outlook.com",
    )
    tlds = ("com", "net", "org", "tr")

    replacements = (
        ("ı", "i"),
        ("ğ", "g"),
        ("ü", "u"),
        ("ş", "s"),
        ("ö", "o"),
        ("ç", "c"),
        ("Ğ", "G"),
        ("Ü", "U"),
        ("Ş", "S"),
        ("İ", "I"),
        ("Ö", "O"),
        ("Ç", "C"),
    )



import re

from string import ascii_uppercase

from .. import BaseProvider, ElementsType

localized = True


def calculate_vin_str_weight(s: str, weight_factor: list) -> int:
    """
    multiply s(str) by weight_factor char by char
    e.g.
    input: s="ABCDE", weight_factor=[1, 2, 3, 4, 5]
    return: A*1 + B*2 + C*3 + D*4 + E*5

    will multiply 0 when len(weight_factor) less than len(s)
    """

    def _get_char_weight(c: str) -> int:
        """A=1, B=2, ...., I=9,
        J=1, K=2, ..., R=9,
        S=2, T=3, ..., Z=9
        """
        if ord(c) <= 64:  # 0-9
            return int(c)
        if ord(c) <= 73:  # A-I
            return ord(c) - 64
        if ord(c) <= 82:  # J-R
            return ord(c) - 73
        # S-Z
        return ord(c) - 81

    res = 0
    for i, c in enumerate(s):
        res += _get_char_weight(c) * weight_factor[i] if i < len(weight_factor) else 0
    return res


class Provider(BaseProvider):
    """Implement default automotive provider for Faker."""

    license_formats: ElementsType = ()

    def license_plate(self) -> str:
        """Generate a license plate."""
        temp = re.sub(
            r"\?",
            lambda x: self.random_element(ascii_uppercase),
            self.random_element(self.license_formats),
        )
        return self.numerify(temp)

    def vin(self) -> str:
        """Generate vin number."""
        vin_chars = "1234567890ABCDEFGHJKLMNPRSTUVWXYZ"  # I, O, Q are restricted
        front_part = self.bothify("????????", letters=vin_chars)
        rear_part = self.bothify("????????", letters=vin_chars)
        front_part_weight = calculate_vin_str_weight(front_part, [8, 7, 6, 5, 4, 3, 2, 10])
        rear_part_weight = calculate_vin_str_weight(rear_part, [9, 8, 7, 6, 5, 4, 3, 2])
        checksum = (front_part_weight + rear_part_weight) % 11
        checksum_char = "X" if checksum == 10 else str(checksum)
        return front_part + checksum_char + rear_part



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "duminica",
        "1": "luni",
        "2": "marti",
        "3": "miercuri",
        "4": "joi",
        "5": "vineri",
        "6": "sambata",
    }

    MONTH_NAMES = {
        "01": "ianuarie",
        "02": "februarie",
        "03": "martie",
        "04": "aprilie",
        "05": "mai",
        "06": "iunie",
        "07": "iulie",
        "08": "august",
        "09": "septembrie",
        "10": "octombrie",
        "11": "noiembrie",
        "12": "decembrie",
    }

    def day_of_week(self):
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self):
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    formats = (
        "{{company_prefix}} {{last_name}}",
        "{{company_prefix}} {{last_name}} {{last_name}}",
        "{{company_prefix}} {{last_name}} {{company_suffix}}",
        "{{company_prefix}} {{last_name}} {{last_name}} {{company_suffix}}",
    )

    # From http://id.wikipedia.org/wiki/Jenis_badan_usaha
    # via
    # https://github.com/fzaninotto/faker/blob/master/src/Faker/Provider/id_ID/Company.php
    company_prefixes = (
        "PT",
        "CV",
        "UD",
        "PD",
        "Perum",
    )

    # From http://id.wikipedia.org/wiki/Jenis_badan_usaha
    # via
    # https://github.com/fzaninotto/faker/blob/master/src/Faker/Provider/id_ID/Company.php
    company_suffixes = (
        "(Persero) Tbk",
        "Tbk",
    )

    def company_prefix(self) -> str:
        return self.random_element(self.company_prefixes)



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """
    Source: http://planet-beruf.de/schuelerinnen/mein-beruf/berufe-von-a-z/
    """

    jobs = [
        "Altenpfleger",
        "Asphaltbauer",
        "Artist",
        "Augenoptiker",
        "Ausbaufacharbeiter",
        "Bäcker",
        "Bankkaufmann",
        "Beamter",
        "Binnenschiffer",
        "Chemikant",
        "Chirurgiemechaniker",
        "Designer",
        "Drogist",
        "Erzieher",
        "Elektroniker",
        "Fachinformatiker",
        "Fleischer",
        "Florist",
        "Forstwirt",
        "Friseur",
        "Informatiker",
        "Programmierer",
        "Techniker",
        "Mathematiker",
        "Koreanistiker",
        "Mechaniker",
        "Pfleger",
        "Polizist",
        "Pilot",
        "Arzt",
        "Krankenschwester",
        "Medizininformatiker",
        "Schornsteinfeger",
        "Winzer",
    ]



from functools import wraps
from typing import Callable, Dict, Tuple, TypeVar

from ..utils import text

T = TypeVar("T")


def slugify(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[T, ...], **kwargs: Dict[str, T]) -> str:
        return text.slugify(fn(*args, **kwargs))

    return wrapper


def slugify_domain(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[T, ...], **kwargs: Dict[str, T]) -> str:
        return text.slugify(fn(*args, **kwargs), allow_dots=True)

    return wrapper


def slugify_unicode(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[T, ...], **kwargs: Dict[str, T]) -> str:
        return text.slugify(fn(*args, **kwargs), allow_unicode=True)

    return wrapper


def lowercase(fn: Callable) -> Callable:
    @wraps(fn)
    def wrapper(*args: Tuple[T, ...], **kwargs: Dict[str, T]) -> str:
        return fn(*args, **kwargs).lower()

    return wrapper



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    DAY_NAMES = {
        "0": "Nedelja",
        "1": "Ponedeljek",
        "2": "Torek",
        "3": "Sreda",
        "4": "Četrtek",
        "5": "Petek",
        "6": "Sobota",
    }

    MONTH_NAMES = {
        "01": "Januar",
        "02": "Februar",
        "03": "Marec",
        "04": "April",
        "05": "Maj",
        "06": "Junij",
        "07": "Julij",
        "08": "Avgust",
        "09": "September",
        "10": "Oktober",
        "11": "November",
        "12": "December",
    }

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



from collections import OrderedDict

from .. import Provider as ColorProvider


class Provider(ColorProvider):
    """Implement color provider for ``ar_PS`` locale."""

    all_colors = OrderedDict(
        (
            ("أزرق أليس", "#F0F8FF"),
            ("أبيض عتيق", "#FAEBD7"),
            ("مائي", "#00FFFF"),
            ("زبرجدي", "#7FFFD4"),
            ("لازوردي", "#F0FFFF"),
            ("صوفي", "#F5F5DC"),
            ("حسائي", "#FFE4C4"),
            ("أسود", "#000000"),
            ("لوزي", "#FFEBCD"),
            ("أزرق", "#0000FF"),
            ("بنفسجي مزرق", "#8A2BE2"),
            ("بني", "#A52A2A"),
            ("خشبية", "#DEB887"),
            ("أزرق كاديتي", "#5F9EA0"),
            ("كرتوزي", "#7FFF00"),
            ("شوكولاتيّ", "#D2691E"),
            ("مرجاني", "#FF7F50"),
            ("قنطريوني", "#6495ED"),
            ("حرير الذرة", "#FFF8DC"),
            ("قرمزي", "#DC143C"),
            ("سيان", "#00FFFF"),
            ("أزرق داكن", "#00008B"),
            ("سيان داكن", "#008B8B"),
            ("عصا الدهب الغامق", "#B8860B"),
            ("رمادي داكن", "#A9A9A9"),
            ("أخضر داكن", "#006400"),
            ("خاكي داكن", "#BDB76B"),
            ("ماجنتا داكن", "#8B008B"),
            ("أخضر زيتوني داكن", "#556B2F"),
            ("برتقالي داكن", "#FF8C00"),
            ("أوركيدي داكن", "#9932CC"),
            ("أحمر داكن", "#8B0000"),
            ("سلموني داكن", "#E9967A"),
            ("أخضر بحري داكن", "#8FBC8F"),
            ("أزرق أردوازي داكن", "#483D8B"),
            ("رمادي لازوردي داكن", "#2F4F4F"),
            ("تركوازي داكن", "#00CED1"),
            ("بنفسج داكن", "#9400D3"),
            ("زهري غامق", "#FF1493"),
            ("أزرق سماوي غامق", "#00BFFF"),
            ("رمادي خافت", "#696969"),
            ("أزرق فريق دودجر", "#1E90FF"),
            ("الطوب شمت", "#B22222"),
            ("أبيض وردي", "#FFFAF0"),
            ("أخضر الغابت", "#228B22"),
            ("فوشي", "#FF00FF"),
            ("رمادي باهت", "#DCDCDC"),
            ("أبيض شبحي", "#F8F8FF"),
            ("ذهبي", "#FFD700"),
            ("ذهبي", "#DAA520"),
            ("رمادي", "#808080"),
            ("أخضر", "#008000"),
            ("أصفر مخضر", "#ADFF2F"),
            ("عسلي", "#F0FFF0"),
            ("وردي فاقع", "#FF69B4"),
            ("قسطلي", "#CD5C5C"),
            ("نيلي", "#4B0082"),
            ("سكري", "#FFFFF0"),
            ("خاكي", "#F0E68C"),
            ("لاڤندر", "#E6E6FA"),
            ("أحمر اللافندر", "#FFF0F5"),
            ("أخضر عشبي", "#7CFC00"),
            ("ليمون شيفوني", "#FFFACD"),
            ("أزرق فاتح", "#ADD8E6"),
            ("مرجاني فاتح", "#F08080"),
            ("أزرق طفولي", "#E0FFFF"),
            ("أصفر ذهبي فاتح ", "#FAFAD2"),
            ("رمادي فاتح", "#D3D3D3"),
            ("أخضر فاتح", "#90EE90"),
            ("وردي فاتح", "#FFB6C1"),
            ("سلموني فاتح", "#FFA07A"),
            ("أخضر بحري فاتح", "#20B2AA"),
            ("سماوي فاتح", "#87CEFA"),
            ("أزرق أردوازي فاتح", "#778899"),
            ("أزرق معدني فاتح", "#B0C4DE"),
            ("أصفر فاتح", "#FFFFE0"),
            ("ليمي", "#00FF00"),
            ("أخضر ليموني", "#32CD32"),
            ("كتاني", "#FAF0E6"),
            ("فوشيا", "#FF00FF"),
            ("كستنائي", "#800000"),
            ("زبرجدي متوسط", "#66CDAA"),
            ("أزرق متوسط", "#0000CD"),
            ("أوركيدي متوسط", "#BA55D3"),
            ("فوشي متوسط", "#9370DB"),
            ("أخضر بحري متوسط", "#3CB371"),
            ("أزرق أردوازي متوسط", "#7B68EE"),
            ("أخضر ربيعي متوسط", "#00FA9A"),
            ("ترموازي متوسط", "#48D1CC"),
            ("أحمر بنفسجي", "#C71585"),
            ("الأزرق متوسط", "#191970"),
            ("نعناعي كريمي", "#F5FFFA"),
            ("الوردي الضبابي", "#FFE4E1"),
            ("موكاسيني", "#FFE4B5"),
            ("أبيض نافاجو", "#FFDEAD"),
            ("كحلي", "#000080"),
            ("رباطي قديم", "#FDF5E6"),
            ("زيتوني", "#808000"),
            ("زيتوني رمادي", "#6B8E23"),
            ("برتقالي", "#FFA500"),
            ("أحمر برتقالي", "#FF4500"),
            ("أوركيدي", "#DA70D6"),
            ("ذهبي باهت", "#EEE8AA"),
            ("أخضر باهت", "#98FB98"),
            ("تركوازي باهت", "#AFEEEE"),
            ("أحمر بنفسجي باهت", "#DB7093"),
            ("بابايا", "#FFEFD5"),
            ("حنطي", "#FFDAB9"),
            ("بيرو", "#CD853F"),
            ("زهري", "#FFC0CB"),
            ("برقوقي", "#DDA0DD"),
            ("أزرق مسحوقي", "#B0E0E6"),
            ("أرجواني", "#800080"),
            ("أحمر", "#FF0000"),
            ("بني وردي", "#BC8F8F"),
            ("أزرق ملكي", "#4169E1"),
            ("بني السرج", "#8B4513"),
            ("سالموني", "#FA8072"),
            ("بني رملي", "#F4A460"),
            ("أخضر بحري", "#2E8B57"),
            ("صدفي", "#FFF5EE"),
            ("سيينا", "#A0522D"),
            ("فضي", "#C0C0C0"),
            ("أزرق سماي", "#87CEEB"),
            ("أزرق أردوازي", "#6A5ACD"),
            ("رمادي معدني", "#708090"),
            ("ثلجي", "#FFFAFA"),
            ("أخضر ربيعي", "#00FF7F"),
            ("أزرق معدني", "#4682B4"),
            ("نطي", "#D2B48C"),
            ("حذفي", "#008080"),
            ("أرجواني", "#D8BFD8"),
            ("طماطمي", "#FF6347"),
            ("تركواز", "#40E0D0"),
            ("بنفسجي", "#EE82EE"),
            ("قمحي", "#F5DEB3"),
            ("أبيض", "#FFFFFF"),
            ("دخاني قمحي", "#F5F5F5"),
            ("أصفر", "#FFFF00"),
            ("أصفر مخضر", "#9ACD32"),
        )
    )

    safe_colors = (
        "أبيض",
        "أخضر",
        "أرجواني",
        "أزرق",
        "أسود",
        "أصفر",
        "حذفي",
        "رمادي",
        "زيتوني",
        "فضي",
        "فوشي",
        "كحلي",
        "كستنائي",
        "ليمي",
        "مائي",
    )



from collections import OrderedDict

from .. import Provider as ColorProvider


class Provider(ColorProvider):
    """Implement color provider for ``az_AZ`` locale."""

    all_colors = OrderedDict(
        (
            ("Akuamarin", "#7FFFD4"),
            ("Azure", "#F0FFFF"),
            ("Bej", "#F5F5DC"),
            ("Qara", "#000000"),
            ("Mavi", "#0000FF"),
            ("Mavi-bənövşəyi", "#8A2BE2"),
            ("Qəhvəyi", "#A52A2A"),
            ("Şokolad", "#D2691E"),
            ("Mərcan", "#FF7F50"),
            ("Tünd mavi", "#00008B"),
            ("Tünd boz", "#A9A9A9"),
            ("Tünd yaşıl", "#006400"),
            ("Tünd Xaki", "#BDB76B"),
            ("Tünd Portağal", "#FF8C00"),
            ("Tünd Qırmızı", "#8B0000"),
            ("Tünd Bənövşəyi", "#9400D3"),
            ("Tünd Çəhrayı", "#FF1493"),
            ("Sönük Boz", "#696969"),
            ("Fuksiya", "#FF00FF"),
            ("Qızıl", "#FFD700"),
            ("Boz", "#808080"),
            ("Yaşıl", "#008000"),
            ("Sarı-yaşıl", "#ADFF2F"),
            ("Xaki", "#F0E68C"),
            ("Lavanda çəhrayı", "#FFF0F5"),
            ("Açıq Mavi", "#ADD8E6"),
            ("Açıq Boz", "#D3D3D3"),
            ("Açıq Yaşıl", "#90EE90"),
            ("Açıq Çəhrayı", "#FFB6C1"),
            ("Açıq Sarı", "#FFFFE0"),
            ("Şabalıd", "#800000"),
            ("Portağal", "#FFA500"),
            ("Narıncı Qırmızı", "#FF4500"),
            ("Solğun Yaşıl", "#98FB98"),
            ("Çəhrayı", "#FFC0CB"),
            ("Qırmızı", "#FF0000"),
            ("Aqua", "#2E8B57"),
            ("Gümüş", "#C0C0C0"),
            ("Firuzə", "#40E0D0"),
            ("Bənövşəyi", "#EE82EE"),
            ("Ağ", "#FFFFFF"),
            ("Sarı", "#FFFF00"),
        )
    )

    safe_colors = (
        "qara",
        "tünd qırmızı",
        "yaşıl",
        "zeytun",
        "bənövşəyi",
        "teal",
        "lime",
        "mavi",
        "gümüşü",
        "boz",
        "sarı",
        "fuksiya",
        "ağ",
    )



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Implement lorem provider for ``ja_JP`` locale."""

    word_connector = ""
    sentence_punctuation = "。"
    word_list = (
        "コミュニティ",
        "隠す",
        "葉",
        "陶器",
        "錯覚",
        "バーゲン",
        "リニア",
        "コーラス",
        "仕上げ",
        "叔父",
        "移動",
        "差別する",
        "極端な",
        "数字",
        "テント",
        "必要",
        "主人",
        "電池",
        "ソース",
        "野球",
        "ストレージ",
        "スキーム",
        "暖かい",
        "ささやき",
        "器官",
        "トリビュート",
        "同行",
        "ジャム",
        "パン",
        "索引",
        "トス",
        "織る",
        "パーセント",
        "拡張",
        "教授",
        "バスケット",
        "創傷",
        "フレーム",
        "明らかにする",
        "フェミニスト",
        "発生する",
        "怒り",
        "ボトル",
        "狐",
        "柔らかい",
        "リフト",
        "バス",
        "雪",
        "画面",
        "パイオニア",
        "マリン",
        "ダイヤモンド",
        "普通の",
        "意図",
        "ヘア",
        "日曜日",
        "プラスチック",
        "衝突",
        "評議会",
        "主婦",
        "保証金",
        "動物",
        "参加する",
        "教会",
        "コミュニケーション",
        "憲法",
        "本質的な",
        "探査",
        "呼ぶ",
        "供給",
        "スペル",
        "再現する",
        "合計",
        "ダッシュ",
        "擁する",
        "知覚",
        "シェービング",
        "コンペ",
        "オークション",
        "細かい",
        "ニュース",
        "癌",
        "トーン",
        "チーズ",
        "反射",
        "ブランチ",
        "コピー",
        "状況",
        "スマッシュ",
        "式",
        "協力",
        "管理する",
        "文言",
        "編組",
        "ジャーナル",
        "腐った",
        "見落とす",
        "ハードウェア",
        "ピック",
        "感謝する",
        "楽しんで",
        "人形",
        "建築",
        "見出し",
        "タワー",
        "ホイール",
        "省略",
        "ログ",
        "助けて",
        "不自然な",
        "出演者",
        "転倒",
        "運",
        "障害",
        "クルー",
        "追放する",
        "月",
        "カレッジ",
        "緩む",
        "分割",
        "欠乏",
        "通行料金",
        "電話",
        "狭い",
        "中央",
        "埋め込む",
        "革新",
        "ブレーキ",
        "コーナー",
        "溝",
        "脊椎",
        "ブラケット",
        "戦略的",
        "尿",
        "血まみれの",
        "尊敬する",
        "催眠術",
        "アクセルペダル",
        "厳しい",
        "サンプル",
        "奨励します",
        "指名",
        "クール",
        "クロス",
        "ヒール",
        "敵対的な",
        "近代化する",
        "部隊",
        "目的",
        "保持する",
        "中世",
        "デッド",
        "ノート",
        "デフォルト",
        "犯罪者",
        "キャビン",
        "副",
        "改善",
        "職人",
        "シュガー",
        "花嫁",
        "倫理",
        "偏差",
        "販売",
        "軸",
        "サラダ",
        "品質",
        "風景",
        "虐待",
        "立派な",
        "ベルベット",
        "ハンマー",
        "キャビネット",
        "トレーナー",
        "リハビリ",
        "サワー",
        "連続",
        "学生",
        "高い",
        "賞賛する",
        "行進",
        "ダニ",
        "証言する",
        "符号",
        "バナー",
        "バケツ",
        "カラム",
        "装置",
        "ヒット",
        "敵",
        "トースト",
        "試してみる",
        "大統領",
        "屋根裏",
        "メニュー",
        "残る",
        "リンク",
        "舗装",
        "インチ",
        "特徴",
        "は",
        "持つ",
        "持っていました",
        "あった",
        "〜",
        "ない",
        "今",
        "今日",
        "持ってる",
        "午前",
        "私",
        "君は",
        "彼",
        "彼女",
        "それ",
        "自体",
        "あなた自身",
        "じぶんの",
        "鉱山",
    )

    parts_of_speech: Dict[str, tuple] = {}



import re
import string

from collections import OrderedDict
from typing import Any, Collection, List, Optional, Sequence, TypeVar, Union

from ..generator import Generator
from ..typing import OrderedDictType
from ..utils.distribution import choices_distribution, choices_distribution_unique

_re_hash = re.compile(r"#")
_re_perc = re.compile(r"%")
_re_dol = re.compile(r"\$")
_re_excl = re.compile(r"!")
_re_at = re.compile(r"@")
_re_qm = re.compile(r"\?")
_re_cir = re.compile(r"\^")

T = TypeVar("T")
ElementsType = Union[Collection[str], Collection[T], OrderedDictType[T, float]]


class BaseProvider:
    __provider__ = "base"
    __lang__: Optional[str] = None
    __use_weighting__ = False

    # Locales supported by Linux Mint from `/usr/share/i18n/SUPPORTED`
    language_locale_codes = {
        "aa": ("DJ", "ER", "ET"),
        "af": ("ZA",),
        "ak": ("GH",),
        "am": ("ET",),
        "an": ("ES",),
        "apn": ("IN",),
        "ar": (
            "AE",
            "BH",
            "DJ",
            "DZ",
            "EG",
            "EH",
            "ER",
            "IL",
            "IN",
            "IQ",
            "JO",
            "KM",
            "KW",
            "LB",
            "LY",
            "MA",
            "MR",
            "OM",
            "PS",
            "QA",
            "SA",
            "SD",
            "SO",
            "SS",
            "SY",
            "TD",
            "TN",
            "YE",
        ),
        "as": ("IN",),
        "ast": ("ES",),
        "ayc": ("PE",),
        "az": ("AZ", "IN"),
        "be": ("BY",),
        "bem": ("ZM",),
        "ber": ("DZ", "MA"),
        "bg": ("BG",),
        "bhb": ("IN",),
        "bho": ("IN",),
        "bn": ("BD", "IN"),
        "bo": ("CN", "IN"),
        "br": ("FR",),
        "brx": ("IN",),
        "bs": ("BA",),
        "byn": ("ER",),
        "ca": ("AD", "ES", "FR", "IT"),
        "ce": ("RU",),
        "ckb": ("IQ",),
        "cmn": ("TW",),
        "crh": ("UA",),
        "cs": ("CZ",),
        "csb": ("PL",),
        "cv": ("RU",),
        "cy": ("GB",),
        "da": ("DK",),
        "de": ("AT", "BE", "CH", "DE", "LI", "LU"),
        "doi": ("IN",),
        "dv": ("MV",),
        "dz": ("BT",),
        "el": ("GR", "CY"),
        "en": (
            "AG",
            "AU",
            "BD",
            "BW",
            "CA",
            "DK",
            "GB",
            "HK",
            "IE",
            "IN",
            "NG",
            "NZ",
            "PH",
            "SG",
            "US",
            "ZA",
            "ZM",
            "ZW",
        ),
        "eo": ("US",),
        "es": (
            "AR",
            "BO",
            "CL",
            "CO",
            "CR",
            "CU",
            "DO",
            "EC",
            "ES",
            "GT",
            "HN",
            "MX",
            "NI",
            "PA",
            "PE",
            "PR",
            "PY",
            "SV",
            "US",
            "UY",
            "VE",
        ),
        "et": ("EE",),
        "eu": ("ES", "FR"),
        "fa": ("IR",),
        "ff": ("SN",),
        "fi": ("FI",),
        "fil": ("PH",),
        "fo": ("FO",),
        "fr": ("CA", "CH", "FR", "LU"),
        "fur": ("IT",),
        "fy": ("NL", "DE"),
        "ga": ("IE",),
        "gd": ("GB",),
        "gez": ("ER", "ET"),
        "gl": ("ES",),
        "gu": ("IN",),
        "gv": ("GB",),
        "ha": ("NG",),
        "hak": ("TW",),
        "he": ("IL",),
        "hi": ("IN",),
        "hne": ("IN",),
        "hr": ("HR",),
        "hsb": ("DE",),
        "ht": ("HT",),
        "hu": ("HU",),
        "hy": ("AM",),
        "ia": ("FR",),
        "id": ("ID",),
        "ig": ("NG",),
        "ik": ("CA",),
        "is": ("IS",),
        "it": ("CH", "IT"),
        "iu": ("CA",),
        "iw": ("IL",),
        "ja": ("JP",),
        "ka": ("GE",),
        "kk": ("KZ",),
        "kl": ("GL",),
        "km": ("KH",),
        "kn": ("IN",),
        "ko": ("KR",),
        "kok": ("IN",),
        "ks": ("IN",),
        "ku": ("TR",),
        "kw": ("GB",),
        "ky": ("KG",),
        "lb": ("LU",),
        "lg": ("UG",),
        "li": ("BE", "NL"),
        "lij": ("IT",),
        "ln": ("CD",),
        "lo": ("LA",),
        "lt": ("LT",),
        "lv": ("LV",),
        "lzh": ("TW",),
        "mag": ("IN",),
        "mai": ("IN",),
        "mg": ("MG",),
        "mhr": ("RU",),
        "mi": ("NZ",),
        "mk": ("MK",),
        "ml": ("IN",),
        "mn": ("MN",),
        "mni": ("IN",),
        "mr": ("IN",),
        "ms": ("MY",),
        "mt": ("MT",),
        "my": ("MM",),
        "nan": ("TW",),
        "nb": ("NO",),
        "nds": ("DE", "NL"),
        "ne": ("NP",),
        "nhn": ("MX",),
        "niu": ("NU", "NZ"),
        "nl": ("AW", "BE", "NL"),
        "nn": ("NO",),
        "nr": ("ZA",),
        "nso": ("ZA",),
        "oc": ("FR",),
        "om": ("ET", "KE"),
        "or": ("IN",),
        "os": ("RU",),
        "pa": ("IN", "PK"),
        "pap": ("AN", "AW", "CW"),
        "pl": ("PL",),
        "ps": ("AF",),
        "pt": ("BR", "PT"),
        "quz": ("PE",),
        "raj": ("IN",),
        "ro": ("RO",),
        "ru": ("RU", "UA"),
        "rw": ("RW",),
        "sa": ("IN",),
        "sat": ("IN",),
        "sc": ("IT",),
        "sd": ("IN", "PK"),
        "se": ("NO",),
        "shs": ("CA",),
        "si": ("LK",),
        "sid": ("ET",),
        "sk": ("SK",),
        "sl": ("SI",),
        "so": ("DJ", "ET", "KE", "SO"),
        "sq": ("AL", "ML"),
        "sr": ("ME", "RS"),
        "ss": ("ZA",),
        "st": ("ZA",),
        "sv": ("FI", "SE"),
        "sw": ("KE", "TZ"),
        "szl": ("PL",),
        "ta": ("IN", "LK"),
        "tcy": ("IN",),
        "te": ("IN",),
        "tg": ("TJ",),
        "th": ("TH",),
        "the": ("NP",),
        "ti": ("ER", "ET"),
        "tig": ("ER",),
        "tk": ("TM",),
        "tl": ("PH",),
        "tn": ("ZA",),
        "tr": ("CY", "TR"),
        "ts": ("ZA",),
        "tt": ("RU",),
        "ug": ("CN",),
        "uk": ("UA",),
        "unm": ("US",),
        "ur": ("IN", "PK"),
        "uz": ("UZ",),
        "ve": ("ZA",),
        "vi": ("VN",),
        "wa": ("BE",),
        "wae": ("CH",),
        "wal": ("ET",),
        "wo": ("SN",),
        "xh": ("ZA",),
        "yi": ("US",),
        "yo": ("NG",),
        "yue": ("HK",),
        "zh": ("CN", "HK", "SG", "TW"),
        "zu": ("ZA",),
    }

    def __init__(self, generator: Any) -> None:
        """
        Base class for fake data providers
        :param generator: `Generator` instance
        """
        self.generator = generator

    def locale(self) -> str:
        """Generate a random underscored i18n locale code (e.g. en_US)."""

        language_code = self.language_code()
        return (
            language_code
            + "_"
            + self.random_element(
                BaseProvider.language_locale_codes[language_code],
            )
        )

    def language_code(self) -> str:
        """Generate a random i18n language code (e.g. en)."""

        return self.random_element(BaseProvider.language_locale_codes.keys())

    def random_int(self, min: int = 0, max: int = 9999, step: int = 1) -> int:
        """Generate a random integer between two integers ``min`` and ``max`` inclusive
        while observing the provided ``step`` value.

        This method is functionally equivalent to randomly sampling an integer
        from the sequence ``range(min, max + 1, step)``.

        :sample: min=0, max=15
        :sample: min=0, max=15, step=3
        """
        return self.generator.random.randrange(min, max + 1, step)

    def random_digit(self) -> int:
        """Generate a random digit (0 to 9)."""

        return self.generator.random.randint(0, 9)

    def random_digit_not_null(self) -> int:
        """Generate a random non-zero digit (1 to 9)."""

        return self.generator.random.randint(1, 9)

    def random_digit_above_two(self) -> int:
        """Generate a random digit above value two (2 to 9)."""

        return self.generator.random.randint(2, 9)

    def random_digit_or_empty(self) -> Union[int, str]:
        """Generate a random digit (0 to 9) or an empty string.

        This method will return an empty string 50% of the time,
        and each digit has a 1/20 chance of being generated.
        """

        if self.generator.random.randint(0, 1):
            return self.generator.random.randint(0, 9)
        else:
            return ""

    def random_digit_not_null_or_empty(self) -> Union[int, str]:
        """Generate a random non-zero digit (1 to 9) or an empty string.

        This method will return an empty string 50% of the time,
        and each digit has a 1/18 chance of being generated.
        """

        if self.generator.random.randint(0, 1):
            return self.generator.random.randint(1, 9)
        else:
            return ""

    def random_number(self, digits: Optional[int] = None, fix_len: bool = False) -> int:
        """Generate a random integer according to the following rules:

        - If ``digits`` is ``None`` (default), its value will be set to a random
          integer from 1 to 9.
        - If ``fix_len`` is ``False`` (default), all integers that do not exceed
          the number of ``digits`` can be generated.
        - If ``fix_len`` is ``True``, only integers with the exact number of
          ``digits`` can be generated.

        :sample: fix_len=False
        :sample: fix_len=True
        :sample: digits=3
        :sample: digits=3, fix_len=False
        :sample: digits=3, fix_len=True
        """
        if digits is None:
            digits = self.random_digit_not_null()
        if digits < 0:
            raise ValueError("The digit parameter must be greater than or equal to 0.")
        if fix_len:
            if digits > 0:
                return self.generator.random.randint(pow(10, digits - 1), pow(10, digits) - 1)
            else:
                raise ValueError("A number of fixed length cannot have less than 1 digit in it.")
        else:
            return self.generator.random.randint(0, pow(10, digits) - 1)

    def random_letter(self) -> str:
        """Generate a random ASCII letter (a-z and A-Z)."""

        return self.generator.random.choice(getattr(string, "letters", string.ascii_letters))

    def random_letters(self, length: int = 16) -> Sequence[str]:
        """Generate a list of random ASCII letters (a-z and A-Z) of the specified ``length``.

        :sample: length=10
        """
        return self.random_choices(
            getattr(string, "letters", string.ascii_letters),
            length=length,
        )

    def random_lowercase_letter(self) -> str:
        """Generate a random lowercase ASCII letter (a-z)."""

        return self.generator.random.choice(string.ascii_lowercase)

    def random_uppercase_letter(self) -> str:
        """Generate a random uppercase ASCII letter (A-Z)."""

        return self.generator.random.choice(string.ascii_uppercase)

    def random_elements(
        self,
        elements: ElementsType[T] = ("a", "b", "c"),  # type: ignore[assignment]
        length: Optional[int] = None,
        unique: bool = False,
        use_weighting: Optional[bool] = None,
    ) -> Sequence[T]:
        """Generate a list of randomly sampled objects from ``elements``.

        Set ``unique`` to ``False`` for random sampling with replacement, and set ``unique`` to
        ``True`` for random sampling without replacement.

        If ``length`` is set to ``None`` or is omitted, ``length`` will be set to a random
        integer from 1 to the size of ``elements``.

        The value of ``length`` cannot be greater than the number of objects
        in ``elements`` if ``unique`` is set to ``True``.

        The value of ``elements`` can be any sequence type (``list``, ``tuple``, ``set``,
        ``string``, etc) or an ``OrderedDict`` type. If it is the latter, the keys will be
        used as the objects for sampling, and the values will be used as weighted probabilities
        if ``unique`` is set to ``False``. For example:

        .. code-block:: python

            # Random sampling with replacement
            fake.random_elements(
                elements=OrderedDict([
                    ("variable_1", 0.5),        # Generates "variable_1" 50% of the time
                    ("variable_2", 0.2),        # Generates "variable_2" 20% of the time
                    ("variable_3", 0.2),        # Generates "variable_3" 20% of the time
                    ("variable_4": 0.1),        # Generates "variable_4" 10% of the time
                ]), unique=False
            )

            # Random sampling without replacement (defaults to uniform distribution)
            fake.random_elements(
                elements=OrderedDict([
                    ("variable_1", 0.5),
                    ("variable_2", 0.2),
                    ("variable_3", 0.2),
                    ("variable_4": 0.1),
                ]), unique=True
            )

        :sample: elements=('a', 'b', 'c', 'd'), unique=False
        :sample: elements=('a', 'b', 'c', 'd'), unique=True
        :sample: elements=('a', 'b', 'c', 'd'), length=10, unique=False
        :sample: elements=('a', 'b', 'c', 'd'), length=4, unique=True
        :sample: elements=OrderedDict([
                        ("a", 0.45),
                        ("b", 0.35),
                       ("c", 0.15),
                       ("d", 0.05),
                   ]), length=20, unique=False
        :sample: elements=OrderedDict([
                       ("a", 0.45),
                       ("b", 0.35),
                       ("c", 0.15),
                       ("d", 0.05),
                   ]), unique=True
        """
        use_weighting = use_weighting if use_weighting is not None else self.__use_weighting__

        if isinstance(elements, dict) and not isinstance(elements, OrderedDict):
            raise ValueError("Use OrderedDict only to avoid dependency on PYTHONHASHSEED (See #363).")

        fn = choices_distribution_unique if unique else choices_distribution

        if length is None:
            length = self.generator.random.randint(1, len(elements))

        if unique and length > len(elements):
            raise ValueError("Sample length cannot be longer than the number of unique elements to pick from.")

        if isinstance(elements, dict):
            if not hasattr(elements, "_key_cache"):
                elements._key_cache = tuple(elements.keys())  # type: ignore

            choices = elements._key_cache  # type: ignore[attr-defined, union-attr]
            probabilities = tuple(elements.values()) if use_weighting else None
        else:
            if unique:
                # shortcut
                return self.generator.random.sample(elements, length)
            choices = elements
            probabilities = None

        return fn(
            tuple(choices),
            probabilities,
            self.generator.random,
            length=length,
        )

    def random_choices(
        self,
        elements: ElementsType[T] = ("a", "b", "c"),  # type: ignore[assignment]
        length: Optional[int] = None,
    ) -> Sequence[T]:
        """Generate a list of objects randomly sampled from ``elements`` with replacement.

        For information on the ``elements`` and ``length`` arguments, please refer to
        :meth:`random_elements() <faker.providers.BaseProvider.random_elements>` which
        is used under the hood with the ``unique`` argument explicitly set to ``False``.

        :sample: elements=('a', 'b', 'c', 'd')
        :sample: elements=('a', 'b', 'c', 'd'), length=10
        :sample: elements=OrderedDict([
                     ("a", 0.45),
                     ("b", 0.35),
                     ("c", 0.15),
                     ("d", 0.05),
                 ])
        :sample: elements=OrderedDict([
                     ("a", 0.45),
                     ("b", 0.35),
                     ("c", 0.15),
                     ("d", 0.05),
                 ]), length=20
        """
        return self.random_elements(elements, length, unique=False)

    def random_element(self, elements: ElementsType[T] = ("a", "b", "c")) -> T:
        """Generate a randomly sampled object from ``elements``.

        For information on the ``elements`` argument, please refer to
        :meth:`random_elements() <faker.providers.BaseProvider.random_elements>` which
        is used under the hood with the ``unique`` argument set to ``False`` and the
        ``length`` argument set to ``1``.

        :sample: elements=('a', 'b', 'c', 'd')
        :sample size=10: elements=OrderedDict([
                     ("a", 0.45),
                     ("b", 0.35),
                     ("c", 0.15),
                     ("d", 0.05),
                 ])
        """

        return self.random_elements(elements, length=1)[0]

    def random_sample(
        self, elements: ElementsType[T] = ("a", "b", "c"), length: Optional[int] = None  # type: ignore[assignment]
    ) -> Sequence[T]:
        """Generate a list of objects randomly sampled from ``elements`` without replacement.

        For information on the ``elements`` and ``length`` arguments, please refer to
        :meth:`random_elements() <faker.providers.BaseProvider.random_elements>` which
        is used under the hood with the ``unique`` argument explicitly set to ``True``.

        :sample: elements=('a', 'b', 'c', 'd', 'e', 'f')
        :sample: elements=('a', 'b', 'c', 'd', 'e', 'f'), length=3
        """
        return self.random_elements(elements, length, unique=True)

    def randomize_nb_elements(
        self,
        number: int = 10,
        le: bool = False,
        ge: bool = False,
        min: Optional[int] = None,
        max: Optional[int] = None,
    ) -> int:
        """Generate a random integer near ``number`` according to the following rules:

        - If ``le`` is ``False`` (default), allow generation up to 140% of ``number``.
          If ``True``, upper bound generation is capped at 100%.
        - If ``ge`` is ``False`` (default), allow generation down to 60% of ``number``.
          If ``True``, lower bound generation is capped at 100%.
        - If a numerical value for ``min`` is provided, generated values less than ``min``
          will be clamped at ``min``.
        - If a numerical value for ``max`` is provided, generated values greater than
          ``max`` will be clamped at ``max``.
        - If both ``le`` and ``ge`` are ``True``, the value of ``number`` will automatically
          be returned, regardless of the values supplied for ``min`` and ``max``.

        :sample: number=100
        :sample: number=100, ge=True
        :sample: number=100, ge=True, min=120
        :sample: number=100, le=True
        :sample: number=100, le=True, max=80
        :sample: number=79, le=True, ge=True, min=80
        """
        if le and ge:
            return number
        _min = 100 if ge else 60
        _max = 100 if le else 140
        nb = int(number * self.generator.random.randint(_min, _max) / 100)
        if min is not None and nb < min:
            nb = min
        if max is not None and nb > max:
            nb = max
        return nb

    def numerify(self, text: str = "###") -> str:
        """Generate a string with each placeholder in ``text`` replaced according
        to the following rules:

        - Number signs ('#') are replaced with a random digit (0 to 9).
        - Percent signs ('%') are replaced with a random non-zero digit (1 to 9).
        - Dollar signs ('$') are replaced with a random digit above two (2 to 9).
        - Exclamation marks ('!') are replaced with a random digit or an empty string.
        - At symbols ('@') are replaced with a random non-zero digit or an empty string.

        Under the hood, this method uses :meth:`random_digit() <faker.providers.BaseProvider.random_digit>`,
        :meth:`random_digit_not_null() <faker.providers.BaseProvider.random_digit_not_null>`,
        :meth:`random_digit_or_empty() <faker.providers.BaseProvider.random_digit_or_empty>`,
        and :meth:`random_digit_not_null_or_empty() <faker.providers.BaseProvider.random_digit_not_null_or_empty>`
        to generate the random values.

        :sample: text='Intel Core i%-%%##K vs AMD Ryzen % %%##X'
        :sample: text='!!! !!@ !@! !@@ @!! @!@ @@! @@@'
        """
        text = _re_hash.sub(lambda x: str(self.random_digit()), text)
        text = _re_perc.sub(lambda x: str(self.random_digit_not_null()), text)
        text = _re_dol.sub(lambda x: str(self.random_digit_above_two()), text)
        text = _re_excl.sub(lambda x: str(self.random_digit_or_empty()), text)
        text = _re_at.sub(lambda x: str(self.random_digit_not_null_or_empty()), text)
        return text

    def lexify(self, text: str = "????", letters: str = string.ascii_letters) -> str:
        """Generate a string with each question mark ('?') in ``text``
        replaced with a random character from ``letters``.

        By default, ``letters`` contains all ASCII letters, uppercase and lowercase.

        :sample: text='Random Identifier: ??????????'
        :sample: text='Random Identifier: ??????????', letters='ABCDE'
        """
        return _re_qm.sub(lambda x: self.random_element(letters), text)

    def bothify(self, text: str = "## ??", letters: str = string.ascii_letters) -> str:
        """Generate a string with each placeholder in ``text`` replaced according to the following rules:

        - Number signs ('#') are replaced with a random digit (0 to 9).
        - Question marks ('?') are replaced with a random character from ``letters``.

        By default, ``letters`` contains all ASCII letters, uppercase and lowercase.

        Under the hood, this method uses :meth:`numerify() <faker.providers.BaseProvider.numerify>` and
        and :meth:`lexify() <faker.providers.BaseProvider.lexify>` to generate random values for number
        signs and question marks respectively.

        :sample: letters='ABCDE'
        :sample: text='Product Number: ????-########'
        :sample: text='Product Number: ????-########', letters='ABCDE'
        """
        return self.lexify(self.numerify(text), letters=letters)

    def hexify(self, text: str = "^^^^", upper: bool = False) -> str:
        """Generate a string with each circumflex ('^') in ``text``
        replaced with a random hexadecimal character.

        By default, ``upper`` is set to False. If set to ``True``, output
        will be formatted using uppercase hexadecimal characters.

        :sample: text='MAC Address: ^^:^^:^^:^^:^^:^^'
        :sample: text='MAC Address: ^^:^^:^^:^^:^^:^^', upper=True
        """
        letters = string.hexdigits[:-6]
        if upper:
            letters = letters.upper()
        return _re_cir.sub(lambda x: self.random_element(letters), text)


class DynamicProvider(BaseProvider):
    def __init__(
        self,
        provider_name: str,
        elements: Optional[List] = None,
        generator: Optional[Any] = None,
    ):
        """
        A faker Provider capable of getting a list of elements to randomly select from,
        instead of using the predefined list of elements which exist in the default providers in faker.

        :param provider_name: Name of provider, which would translate into the function name e.g. faker.my_fun().
        :param elements: List of values to randomly select from
        :param generator: Generator object. If missing, the default Generator is used.

        :example:
        >>>from faker import Faker
        >>>from faker.providers import DynamicProvider

        >>>medical_professions_provider = DynamicProvider(
        >>>     provider_name="medical_profession",
        >>>     elements=["dr.", "doctor", "nurse", "surgeon", "clerk"],
        >>>)
        >>>fake = Faker()
        >>>fake.add_provider(medical_professions_provider)

        >>>fake.medical_profession()
        "dr."

        """

        if not generator:
            generator = Generator()
        super().__init__(generator)
        if provider_name.startswith("__"):
            raise ValueError("Provider name cannot start with __ as it would be ignored by Faker")

        self.provider_name = provider_name

        self.elements = []
        if elements:
            self.elements = elements

        setattr(self, provider_name, self.get_random_value)  # Add a method for the provider_name value

    def add_element(self, element: str) -> None:
        """Add new element."""
        self.elements.append(element)

    def get_random_value(self, use_weighting: bool = True) -> Any:
        """Returns a random value for this provider.

        :param use_weighting: boolean option to use weighting. Defaults to True
        """
        if not self.elements or len(self.elements) == 0:
            raise ValueError("Elements should be a list of values the provider samples from")

        return self.random_elements(self.elements, length=1, use_weighting=use_weighting)[0]



import re

from typing import Pattern

from faker.providers.automotive import calculate_vin_str_weight
from faker.providers.automotive.de_DE import Provider as DeDeAutomotiveProvider
from faker.providers.automotive.es_ES import Provider as EsEsAutomotiveProvider
from faker.providers.automotive.ro_RO import Provider as RoRoAutomotiveProvider
from faker.providers.automotive.ru_RU import Provider as RuRuAutomotiveProvider
from faker.providers.automotive.sk_SK import Provider as SkSkAutomotiveProvider
from faker.providers.automotive.tr_TR import Provider as TrTrAutomotiveProvider
from faker.providers.automotive.uk_UA import Provider as UkUaAutomotiveProvider


class _SimpleAutomotiveTestMixin:
    """Use this test mixin for simple license plate validation"""

    def perform_extra_checks(self, license_plate, match):
        pass

    def test_license_plate(self, faker, num_samples):
        for _ in range(num_samples):
            license_plate = faker.license_plate()
            match = self.license_plate_pattern.fullmatch(license_plate)
            assert match is not None
            self.perform_extra_checks(license_plate, match)

    def test_vin(self, faker, num_samples):
        for _ in range(num_samples):
            vin_number = faker.vin()
            # length check: 17
            assert len(vin_number) == 17

            # verify checksum: vin_number[8]
            front_part_weight = calculate_vin_str_weight(vin_number[:8], [8, 7, 6, 5, 4, 3, 2, 10])
            rear_part_weight = calculate_vin_str_weight(vin_number[9:], [9, 8, 7, 6, 5, 4, 3, 2])
            checksum = (front_part_weight + rear_part_weight) % 11
            checksum_str = "X" if checksum == 10 else str(checksum)
            assert vin_number[8] == checksum_str


class TestArBh(_SimpleAutomotiveTestMixin):
    """Test ar_BH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{6}")


class TestAzAz(_SimpleAutomotiveTestMixin):
    """Test az_AZ automotive provider methods"""

    license_plate_pattern = re.compile(r"\d{2}-[A-Z]{2}-\d{3}")


class TestSkSk(_SimpleAutomotiveTestMixin):
    """Test sk_SK automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(?P<prefix>[A-Z]{2})\d{3}[A-Z]{2}")

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in SkSkAutomotiveProvider.license_plate_prefix


class TestPtBr(_SimpleAutomotiveTestMixin):
    """Test pt_BR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{1}[A-Z]{1}\d{2}")


class TestPtPt(_SimpleAutomotiveTestMixin):
    """Test pt_PT automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"\d{2}-\d{2}-[A-Z]{2}|" r"\d{2}-[A-Z]{2}-\d{2}|" r"[A-Z]{2}-\d{2}-\d{2}|" r"[A-Z]{2}-\d{2}-[A-Z]{2}",
    )


class TestHeIl(_SimpleAutomotiveTestMixin):
    license_plate_pattern: Pattern = re.compile(r"(\d{3}-\d{2}-\d{3})|(\d{2}-\d{3}-\d{2})")


class TestHuHu(_SimpleAutomotiveTestMixin):
    """Test hu_HU automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{3}")


class TestDeDe(_SimpleAutomotiveTestMixin):
    """Test de_DE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"(?P<prefix>[A-Z\u00D6\u00DC]{1,3})-[A-Z]{1,2}-[1-9]{1,4}",
        re.UNICODE,
    )

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in DeDeAutomotiveProvider.license_plate_prefix


class TestSvSe(_SimpleAutomotiveTestMixin):
    """Test sv_SE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3} \d{2}[\dA-Z]")


class TestPlPl:
    def test_License_plate(self, faker, num_samples):
        pattern: Pattern = re.compile(r"{patterns}".format(patterns="|".join(faker.license_plate_regex_formats())))
        for _ in range(num_samples):
            plate = faker.license_plate()
            assert pattern.fullmatch(plate)


class TestEnPh(_SimpleAutomotiveTestMixin):
    """Test en_PH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4,5}|[A-Z]{3}\d{3,4}")
    motorcycle_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4,5}")
    automobile_pattern: Pattern = re.compile(r"[A-Z]{3}\d{3,4}")

    def test_motorcycle_plate(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.motorcycle_pattern.match(faker.motorcycle_license_plate())

    def test_automobile_plate(self, faker, num_samples):
        for _ in range(num_samples):
            assert self.automobile_pattern.match(faker.automobile_license_plate())

    def test_protocol_plate(self, faker, num_samples):
        for _ in range(num_samples):
            protocol_plate = faker.protocol_license_plate()
            assert int(protocol_plate) != 15 and 1 <= int(protocol_plate) <= 17


class TestFilPh(TestEnPh):
    """Test fil_PH automotive provider methods"""

    pass


class TestTlPh(TestEnPh):
    """Test tl_PH automotive provider methods"""

    pass


class TestRuRu(_SimpleAutomotiveTestMixin):
    """Test ru_RU automotive provider methods"""

    _plate_letters = "".join(RuRuAutomotiveProvider.license_plate_letters)
    license_plate_pattern: Pattern = re.compile(
        r"(?:"
        r"(?P<private_plate_prefix>[{0}]\d\d\d[{0}][{0}])|"
        r"(?P<public_transport_plate_prefix>[{0}][{0}]\d\d\d)|"
        r"(?P<trailer_plate_prefix>[{0}][{0}]\d\d\d\d)|"
        r"(?P<police_plate_prefix>[{0}]\d\d\d\d)|"
        r"(?P<military_plate_prefix>\d\d\d\d[{0}][{0}])|"
        r"(?P<plate_number_special>00\dCD\d|00\dD\d\d\d|00\dT\d\d\d)"
        r") (?P<plate_suffix>.*)".format(_plate_letters),
    )

    def perform_extra_checks(self, license_plate, match):
        plate_suffix = match.group("plate_suffix")
        assert plate_suffix in RuRuAutomotiveProvider.license_plate_suffix

    def test_vehicle_category(self, faker, num_samples):
        for _ in range(num_samples):
            vehicle_category = faker.vehicle_category()
            assert isinstance(vehicle_category, str)
            assert vehicle_category in RuRuAutomotiveProvider.vehicle_categories


class TestFrFr(_SimpleAutomotiveTestMixin):
    """Test fr_FR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{3}-[A-Z]{3}-\d{2}|[A-Z]{2}-\d{3}-[A-Z]{2}")


class TestItIt(_SimpleAutomotiveTestMixin):
    """Test it_IT automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{3}[A-Z]{2}")


class TestNoNo(_SimpleAutomotiveTestMixin):
    """Test no_NO automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2} \d{5}")


class TestEsCo(_SimpleAutomotiveTestMixin):
    """Test es_CO automotive provider methods"""

    license_plate_pattern = re.compile(r"[A-Z]{3}\d{3}|[A-Z]{3}\d{2}[A-Z]|T\d{4}|[A-Z]{2}\d{4}|R\d{5}|S\d{5}")


class TestEsEs:
    """Test es_ES automotive provider methods"""

    new_format_pattern: Pattern = re.compile(r"\d{4}\s[A-Z]{3}")
    old_format_pattern: Pattern = re.compile(r"(?P<province_prefix>[A-Z]{1,2})\s\d{4}\s[A-Z]{2}")

    def test_plate_new_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_unified()
            assert isinstance(plate, str)
            assert self.new_format_pattern.match(plate)

    def test_plate_old_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_province()
            assert isinstance(plate, str)
            match = self.old_format_pattern.match(plate)
            assert match
            assert match.group("province_prefix") in EsEsAutomotiveProvider.province_prefix

    def test_plate_old_format_explicit_province_prefix(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_by_province(province_prefix="CA")
            assert isinstance(plate, str)
            assert self.old_format_pattern.match(plate)
            assert plate[:2] == "CA"

    def test_plate_format(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate()
            assert isinstance(plate, str)
            assert self.new_format_pattern.match(plate) or self.old_format_pattern.match(plate)


class TestThTh(_SimpleAutomotiveTestMixin):
    """Test th_TH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"(\d [ก-ฮ]{2} \d{1,4})|"  # car
        r"([ก-ฮ]{2} \d{1,4})|"  # car
        r"([ก-ฮ]{3} \d{1,3})|"  # motorcycle
        r"(\d{2}-\d{4})",  # truck
    )


class TestTrTr(_SimpleAutomotiveTestMixin):
    """Test tr_TR automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"\d{2} [A-Z] \d{4}|"
        r"\d{2} [A-Z] \d{5}|"
        r"\d{2} [A-Z]{2} \d{3}|"
        r"\d{2} [A-Z]{2} \d{4}|"
        r"\d{2} [A-Z]{3} \d{2}|"
        r"\d{2} [A-Z]{3} \d{3}",
    )

    def perform_extra_checks(self, license_plate, match):
        [city_code, letters, _] = license_plate.split(" ")
        assert int(city_code) in range(1, 82)
        assert all(letter in TrTrAutomotiveProvider.ascii_uppercase_turkish for letter in letters)


class TestRoRo(_SimpleAutomotiveTestMixin):
    """Test ro_RO automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(?P<prefix>[A-Z]{1,2})-\d{2,3}-[A-Z]{3}")

    def perform_extra_checks(self, license_plate, match):
        assert match.group("prefix") in RoRoAutomotiveProvider.license_plate_prefix


class TestElGr(_SimpleAutomotiveTestMixin):
    """Test el_GR automotive provider methods"""

    license_plate_pattern = re.compile(r"^(?P<prefix>[A-Z]{2,3}) \d{4}$")


class TestNlNl(_SimpleAutomotiveTestMixin):
    """Test nl_NL automotive provider methods"""

    license_plate_car_pattern = re.compile(
        r"\d{2}-[BDFGHJKLNPRSTVXZ][A-Z]-[A-Z]{2}|"
        r"\d{2}-[BDFGHJKLNPRSTVXZ][A-Z]{2}-\d|"
        r"\d-[KSTVXZ][A-Z]{2}-\d{2}|"
        r"[BDFGHJKLNPRSTVXZ][A-Z]-\d{3}-[A-Z]|"
        r"[BDFGHJKLNPRSTVXZ]-\d{3}-[A-Z]{2}",
    )

    license_plate_motorbike_pattern = re.compile(
        r"M[A-Z]-[A-Z]{2}-\d{2}|" r"\d{2}-M[A-Z]-[A-Z]{2}",
    )

    license_plate_pattern = re.compile(
        license_plate_car_pattern.pattern + "|" + license_plate_motorbike_pattern.pattern,
    )

    def test_plate_car(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_car()
            assert isinstance(plate, str)
            assert self.license_plate_car_pattern.match(plate)

    def test_plate_motorbike(self, faker, num_samples):
        for _ in range(num_samples):
            plate = faker.license_plate_motorbike()
            assert isinstance(plate, str)
            assert self.license_plate_motorbike_pattern.match(plate)


class TestViVn(_SimpleAutomotiveTestMixin):
    """Test vi_VN automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"\d{2}[ABCDĐEFGHKLMNPSTUVXYZ]-\d{5}")


class TestFiFi(_SimpleAutomotiveTestMixin):
    """Test fi_FI automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{3}-\d{3}")


class TestSqAl(_SimpleAutomotiveTestMixin):
    """Test sq_AL automotive providers methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2} \d{3}[A-Z]{2}")


class TestDeCh(_SimpleAutomotiveTestMixin):
    """Test de_CH automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}-\d{1,3}\s?\d{0,3}")


class TestNlBe(_SimpleAutomotiveTestMixin):
    """Test nl_BE automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"(\d{3}-[A-Z]{3})|" r"([A-Z]{3}-\d{3})|" r"([1-2]-[A-Z]{3}-\d{3})")


class TestZhCn(_SimpleAutomotiveTestMixin):
    """Test zh_CN automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(r"^[京津冀晋蒙辽吉黑沪苏浙皖闽赣鲁豫鄂湘粤桂琼川贵云渝藏陕甘青宁新]{1}[A-Z]{1}-[A-Z0-9]{5}")


class TestZhTw(_SimpleAutomotiveTestMixin):
    """Test zh_TW automotive provider methods"""

    license_plate_pattern: Pattern = re.compile(
        r"([A-Z]{2}-\d{4})|"  # prior 2012 v1
        r"(\d{4}-[A-Z]{2})|"  # prior 2012 v2
        r"([A-Z]{3}-\d{4})|"  # new format since 2014
        r"([A-Z]{3}-\d{3})",  # commercial cars since 2012
    )


class TestUkUa(_SimpleAutomotiveTestMixin):
    license_plate_pattern: Pattern = re.compile(r"[A-Z]{2}\d{4}[A-Z]{2}")

    def perform_extra_checks(self, license_plate, match):
        assert license_plate[-2:] in UkUaAutomotiveProvider.license_plate_suffix

    def test_temporary_plate(self, faker, num_samples):
        pattern = r"\d{2} [A-Z]{2}\d{4}"

        for _ in range(num_samples):
            temporary = faker.license_plate(temporary_plate=True)
            match = re.search(pattern, temporary)
            assert match is not None

    def test_diplomatic_plate(self, faker, num_samples):
        pattern = r"(CDP \d{3})|(DP|S) \d{3} \d{3}"

        for _ in range(num_samples):
            temporary = faker.diplomatic_license_plate()
            match = re.search(pattern, temporary)
            assert match is not None

    def test_prefix(self, faker):
        for _ in range(10):
            temporary = faker.plate_letter_prefix(region_name="Lviv")
            assert len(temporary) == 2
            assert temporary in UkUaAutomotiveProvider.license_region_data.get("Lviv")[0]

    def test_region_code(self, faker):
        assert "14" == faker.plate_region_code(region_name="Lviv")



from typing import Dict

from .. import Provider as LoremProvider
from ..nl_BE import Provider as LoremProviderNL_BE


class Provider(LoremProvider):
    """Implement lorem provider for ``nl_NL`` locale.

    Source: https://nl.wiktionary.org/wiki/WikiWoordenboek:Lijst_met_1000_basiswoorden
    """

    word_list = LoremProviderNL_BE.word_list
    parts_of_speech: Dict[str, tuple] = {}
